/*
 * Tirititran es un juego de naves en 2D ambientado en Cádiz
 * Copyright (C) 2004  Javier Pérez Pacheco
 *
 * Este juego tienen licencia Creative Commons y se permite
 * su modificacion y utilizacion libremente siempre y cuando se
 * cite al autor original y se comparta con la misma licencia.
 * No se permite su uso comercial.
 * 
 * Para mas informacion visite la web:
 * http://creativecommons.org/licenses/by-nc-sa/2.0/es/
 *
 * PROGRAMADOR
 * Javier Pérez Pacheco
 * Cadiz (Spain)
 * javi.pacheco@terra.es
 *
 * GRAFISMO Y 3D
 * Jesus Carrasco
 * Cadiz (Spain)
 * carrasco.carrasco@gmail.com
 *
 * MUSICA Y GRAFISMO
 * Shano Lores
 * Cadiz (Spain)
 * shano@mail.ono.es
 *
 */

#ifndef DISPAROSENEMIGOS_H_
#define DISPAROSENEMIGOS_H_

#include <SDL.h>
#include "defines.h"
#include "elementos.h"
#include "sonido.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <vector>

using namespace std;

class Elemento;
class Explosion;
class Jugador;
class Frame;
class Audio;
	
typedef struct
{
	int tipoDisparo;
	int movDisparo;
	int x;
	int y;
	int dirigirX;
	int posTorreta;
	int ciclo;
} DisparoRetardado;

class DisparosEnemigos {
	
private:
	vector<DisparoRetardado> disp_retardado;
	Elemento* disp_peq[MAXDISP_PEQ];
	Elemento* disp_gr[MAXDISP_GR];
	Elemento* disp_gr_azul[MAXDISP_GR];
	Elemento* disp_rot[MAXDISP_ROT];
	Elemento* pompa_gr[MAXDISP_POMPA];
	Elemento* pompa_peq[MAXDISP_POMPA];
	int ciclos;
	int elementoLibre (int ele);
	float arSeno[16], arCoseno[16];
	Jugador *j;

public:
	DisparosEnemigos(Jugador *jug);
	void crearDisparo(int tipoDisparo, int movDisparo,float x, float y, float dirigirX, int posTorreta);
	void crearDisparoRetardado (int tipoDisparo, int movDisparo, float x, float y, float dirigirX, int posTorreta, int ciclosRetardo);
	void lanzarDisparoRetardado ();
	// dirigirX es para tipo de disparo DISPARO_DIRIGIDO
	// posTorreta es para tipo de disparo DISPARO_GRADOALJUGADOR
	void moverDisparo (SDL_Surface *sc);
	void vaciarDisparoRetardado() { disp_retardado.resize(0); }
	bool colisionConJugador();
	void colisionConDisparosJugador(Elemento *b, Explosion *exp, Audio *son);
	void iniciar();
	void unLoad();
	float getSeno(int pos) { return arSeno[pos]; }
	float getCoseno(int pos) { return arCoseno[pos]; }
};


#endif /* DISPAROSENEMIGOS_H_ */
