/*
 * Tirititran es un juego de naves en 2D ambientado en Cádiz
 * Copyright (C) 2004  Javier Pérez Pacheco
 *
 * Este juego tienen licencia Creative Commons y se permite
 * su modificacion y utilizacion libremente siempre y cuando se
 * cite al autor original y se comparta con la misma licencia.
 * No se permite su uso comercial.
 * 
 * Para mas informacion visite la web:
 * http://creativecommons.org/licenses/by-nc-sa/2.0/es/
 *
 * PROGRAMADOR
 * Javier Pérez Pacheco
 * Cadiz (Spain)
 * javi.pacheco@terra.es
 *
 * GRAFISMO Y 3D
 * Jesus Carrasco
 * Cadiz (Spain)
 * carrasco.carrasco@gmail.com
 *
 * MUSICA Y GRAFISMO
 * Shano Lores
 * Cadiz (Spain)
 * shano@mail.ono.es
 *
 */

#include "elementos.h"

/*****************************
**
** CLASE FRAME
**
******************************/

SDL_RWops* Frame::RWFromRar(char *fileRar, char *file)
{
  char *data_ptr;
  unsigned long data_size;
  SDL_RWops *rwops;

  if(urarlib_get(&data_ptr, &data_size, file, fileRar, "password")) 
  {
	rwops = SDL_RWFromMem(data_ptr, data_size);
    //if(data_ptr != NULL) free(data_ptr);
  } else 
  { 
    printf("Error al descomprimir! %s - %s\n", fileRar, file); 
  }
  return rwops;
}

void Frame::load (char *path) {
	SDL_Surface *tmpImg;
	tmpImg = IMG_Load (path);
	if ( tmpImg == NULL) {
		printf ("No se cargo la imagen: %s\n", path);
	}
	img = SDL_DisplayFormatAlpha(tmpImg);
	SDL_FreeSurface (tmpImg);	
}

void Frame::load (char *path, SDL_Surface *sc) {
	SDL_Surface *tmpImg;
	tmpImg = IMG_Load (path);
	if ( tmpImg == NULL) {
		printf ("No se cargo la imagen: %s\n", path);
	}
	//img = SDL_ConvertSurface ( tmpImg, sc->format, SDL_SWSURFACE);
	img = SDL_DisplayFormatAlpha(tmpImg);
	SDL_FreeSurface (tmpImg);	
}

void Frame::loadFromRAR (char *fileRar, char *path) {
	SDL_Surface *tmpImg;
	SDL_RWops *rw;
	rw = RWFromRar(fileRar, path);
	tmpImg = IMG_Load_RW(rw, 0);
	if ( tmpImg == NULL) {
		printf ("No se cargo la imagen: %s\n", path);
	}
	img = SDL_DisplayFormatAlpha(tmpImg);
	SDL_FreeSurface (tmpImg);	
}

void Frame::loadFromRAR (char *fileRar, char *path, SDL_Surface *sc) {
	SDL_Surface *tmpImg;
	SDL_RWops *rw;
	rw = RWFromRar(fileRar, path);
	tmpImg = IMG_Load_RW(rw, 0);
	if ( tmpImg == NULL) {
		printf ("No se cargo la imagen: %s\n", path);
	}
	//img = SDL_ConvertSurface ( tmpImg, sc->format, SDL_SWSURFACE);
	img = SDL_DisplayFormatAlpha(tmpImg);
	SDL_FreeSurface (tmpImg);	
}

void Frame::loadSurface (SDL_Surface *surface) {
	img = surface;
}

void Frame::loadSurface (SDL_Surface *surface, SDL_Surface *sc) {
	img = SDL_ConvertSurface ( surface, sc->format, SDL_SWSURFACE);
	//img = surface;
}

void Frame::unLoad() {
	SDL_FreeSurface (img);	
}

/*****************************
**
** CLASE ELEMENTO
**
******************************/

Elemento::Elemento() {
	fantasma = 0;
	impactosParaMorir = 1;
	actualImpactos = 0;
	int nf=1;
	actualFrame = 0;
	tipoCoordenadas = 0;
	rebote = 1;
	ciclosEntreFrames = 0;
	tipoRotacion = ROTACION_PENDULAR;
	ciclo = 0;
	parpadeo = 0;
	estadoParpadeo = 1;
	cortinilla = 0;
	posXNueva = 0;
	posYNueva = 0;
	dx = 0;
	dy = 0;
	
	sprite = new Frame[nf];
	transparente = new Uint32[nf];
	nFrames = nf;
	cont = 0;
	activo = 0;
	oldX = 0;
	oldY = 0;
	alpha = 255;
	
	align = ALIGN_LEFT;
	valign = VALIGN_TOP;
}

Elemento::Elemento (int nf) {
	fantasma = 0;
	impactosParaMorir = 1;
	actualImpactos = 0;
	actualFrame = 0;
	tipoCoordenadas = 0;
	rebote = 1;
	ciclosEntreFrames = 0;
	tipoRotacion = ROTACION_PENDULAR;
	ciclo = 0;
	parpadeo = 0;
	estadoParpadeo = 1;
	cortinilla = 0;
	posXNueva = 0;
	posYNueva = 0;
	dx = 0;
	dy = 0;
	
	sprite = new Frame[nf];
	transparente = new Uint32[nf];
	nFrames = nf;
	cont = 0;	
	activo = 0;
	oldX = 0;
	oldY = 0;
	alpha = 255;
	
	align = ALIGN_LEFT;
	valign = VALIGN_TOP;
}

Elemento::Elemento (int nf, int ni) {
	fantasma = 0;
	actualImpactos = 0;
	actualFrame = 0;
	tipoCoordenadas = 0;
	ciclosEntreFrames = 0;
	tipoRotacion = ROTACION_PENDULAR;
	ciclo = 0;
	parpadeo = 0;
	estadoParpadeo = 1;
	cortinilla = 0;
	posXNueva = 0;
	posYNueva = 0;
	dx = 0;
	dy = 0;
	
	sprite = new Frame[nf];
	transparente = new Uint32[nf];
	nFrames = nf;
	impactosParaMorir = ni;
	cont = 0;	
	activo = 0;
	oldX = 0;
	oldY = 0;
	alpha = 255;
	
	align = ALIGN_LEFT;
	valign = VALIGN_TOP;
}

bool Elemento::fueraLimitesDePantalla() {
	if ( ( getY() < getLimiteSup() ) || ( getY() > getLimiteInf() ) || ( getX() < getLimiteIzq() ) || ( getX() > getLimiteDer() ) ){ 
		return 1;
	} else {
		return 0;
	}
}

void Elemento::unLoad() {
	int i;
	for (i=0; i<nFrames-1; i++)
		sprite[i].unLoad();
}

void Elemento::addFrame (Frame frame) {
	if (cont<nFrames) {
		transparente[cont] = SDL_GetPixel (frame.img, 0,0); // guarda el valor transparente del frame
		sprite[cont]=frame;
		cont++;
	}
}

void Elemento::addFrame (int pos, Frame frame) {
	if (pos<nFrames) {
		transparente[pos] = SDL_GetPixel (frame.img, 0,0); // guarda el valor transparente del frame
		sprite[pos]=frame;
	}
}

void Elemento::selFrame (int nf) {
	if (nf<=nFrames) {
		anteriorFrame = actualFrame;
		actualFrame = nf;
	}
}

void Elemento::selSiguienteFrame () {
	if (actualFrame<cont-1) {
		actualFrame++;
	} else {
		actualFrame = 0;
	}
}

void Elemento::selSiguienteFrame_Rebote () {
	if (actualFrame>cont-2) {
		rebote=-1;
	}
	if (actualFrame<1) {
		rebote=1;
	}
	actualFrame += rebote;
}

void Elemento::selSiguienteFrame_ReboteHasta (int hasta) {
	if (actualFrame>hasta-2) {
		rebote=-1;
	}
	if (actualFrame<1) {
		rebote=1;
	}
	actualFrame += rebote;
}

void Elemento::conmutaFantasma () {
	if ( fantasma == 0 ) {
		fantasma = 1;
	} else {
		fantasma = 0;
	}	
}

void Elemento::mover () { 
	if (tipoCoordenadas == COORDENADASPOLARES) {
		addX( polarDistancia*polarCoseno );
		addY( polarDistancia*polarSeno );
	} else if (tipoCoordenadas == PORPOSICION) {
		if (posX!=posXNueva) { 
			addX( (posXNueva-posX)/3 ); 
		}
		if (posY!=posYNueva) { 
			addY( (int)((posYNueva-posY)/3) ); 
		}
	} else {
		addX( dx ); 
		addY( dy ); 
	}
}

void Elemento::dibujar (SDL_Surface *superficie) {
	ciclo++;
	
	if (parpadeo >0) {
		if (ciclo%parpadeo==0) {
			if (estadoParpadeo==0) {
				estadoParpadeo = 1;
			} else {
				estadoParpadeo = 0;
			}
		}
	}
	if ( (parpadeo >0) && (estadoParpadeo==0) ) {
		return;
	}
		
	SDL_Rect src, dest;
	
	if (cortinilla>0) {
		src.x = 0;
		src.y = 0;
		src.w = (getAncho()/maxCortinilla) * (maxCortinilla - cortinilla);
		src.h = (getAlto()/maxCortinilla) * (maxCortinilla - cortinilla);
		cortinilla--;
	} else {
		src.x = 0;
		src.y = 0;
		src.w = getAncho();
		src.h = getAlto();
	}
	
	if (nFrames > 1) {
		if (ciclosEntreFrames>0) {
			if (ciclo%ciclosEntreFrames==0) {
				if (tipoRotacion==ROTACION_CIRCULAR) {
					selSiguienteFrame();
				} else {
					selSiguienteFrame_Rebote();
				}
			}
		}
	}
	
	switch (align)
	{
		case  1:
			dest.x = (int)posX - getAncho()/2;
			break;
		case  2:
			dest.x = (int)posX - getAncho();
			break;
		default:
			dest.x = (int)posX;
			break;
	}
	
	switch (valign)
	{
		case  1:
			dest.y = (int)posY - getAlto()/2;
			break;
		case  2:
			dest.y = (int)posY - getAlto();
			break;
		default:
			dest.y = (int)posY;
			break;
	}
	
  	if(alpha != 255) {
		SDL_Surface* sdl_surface_copy = SDL_CreateRGBSurface (sprite[actualFrame].img->flags,
										sprite[actualFrame].img->w, sprite[actualFrame].img->h, sprite[actualFrame].img->format->BitsPerPixel,
										sprite[actualFrame].img->format->Rmask, sprite[actualFrame].img->format->Gmask,
										sprite[actualFrame].img->format->Bmask,
										0);
		int colorkey = SDL_MapRGB(sdl_surface_copy->format, 255, 0, 255);
		SDL_FillRect(sdl_surface_copy, NULL, colorkey);
		SDL_SetColorKey(sdl_surface_copy, SDL_SRCCOLORKEY, colorkey);
	
	
		SDL_BlitSurface(sprite[actualFrame].img, NULL, sdl_surface_copy, NULL);
		SDL_SetAlpha(sdl_surface_copy ,SDL_SRCALPHA,alpha);
		SDL_BlitSurface (sdl_surface_copy, &src, superficie, &dest);
		SDL_FreeSurface (sdl_surface_copy);
	} else {
		SDL_BlitSurface (sprite[actualFrame].img, &src, superficie, &dest);
	}

}


int Elemento::colision (Elemento *elem) {
	int w1,h1,w2,h2,x1,y1,x2,y2;
	
	w1 = (int)getAncho();
	h1 = (int)getAlto();
	w2 = (int)elem->getAncho();
	h2 = (int)elem->getAlto();
	x1 = (int)getX();
	y1 = (int)getY();
	x2 = (int)elem->getX();
	y2 = (int)elem->getY();
	
	return SDL_Collide( getSurface() , x1 , y1, getTransparente() , elem->getSurface(), x2 , y2, elem->getTransparente());

}

Uint32 Elemento::SDL_GetPixel(SDL_Surface *surface, int x, int y) {
	int bpp = surface->format->BytesPerPixel;
	/* Here p is the address to the pixel we want to retrieve */
	Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

	switch(bpp)
	{
		case(1):
		return *p;

		case(2):
		return *(Uint16 *)p;

		case(3):
			if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
				return p[0] << 16 | p[1] << 8 | p[2];
			else
				return p[0] | p[1] << 8 | p[2] << 16;

		case(4):
			return *(Uint32 *)p;

		default:
			return 0;       /* shouldn't happen, but avoids warnings */
	}
}

/*This surfaces must not have any transparency mapped to them at all*/
int Elemento::SDL_Collide( SDL_Surface *SA , int Ax1 , int Ay1 , Uint32 t1,
		 SDL_Surface *SB , int Bx1 , int By1, Uint32 t2	)
{

	/*Box A;
	Box B;*/

	/*Set right and bottom co-ordinates*/
	int Ax2 = Ax1 + SA->w;
	int Ay2 = Ay1 + SA->h;

	int Bx2 = Bx1 + SB->w;
	int By2 = By1 + SB->h;

	if( Bx2 < Ax1 )		return 0;	/*Just checking if their*/
	if( Bx1 > Ax2 )		return 0;	/*Bounding boxes even touch*/
						/*Write down on paper if*/
	if( By2 < Ay1 )		return 0;	/*Don't make sense*/
	if( By1 > Ay2 )		return 0;


	int bx1 = ( Ax1 > Bx1 ) ? Ax1 : Bx1;
	int bx2 = ( Ax2 < Bx2 ) ? Ax2 : Bx2;

	int by1 = ( Ay1 > By1 ) ? Ay1 : By1;
	int by2 = ( Ay2 < By2 ) ? Ay2 : By2;


	for( int y = by1 ; y < by2 ; y++ )
	{
		for( int x = bx1 ; x < bx2 ; x++ )
		{
		if (   ( SDL_GetPixel( SA , x - Ax1 , y - Ay1 ) != t1 ) 
			&& ( SDL_GetPixel( SB , x - Bx1 , y - By1 ) != t2 )  ) { 
				return 1;
			}
		}
	}

	return 0;	/*We searched whole area of overlap and their was No Collision at All*/

}

/*****************************
**
** CLASE BALON
**
******************************/

Balon::Balon()  : Elemento () {
	iniciar();
}

Balon::Balon(int nf)  : Elemento (nf) {
	iniciar();
}

void Balon::mover() {
	ciclos++;
	addX(velHorizontal);
	addY(velVertical);
}

void Balon::iniciar() {
	ciclos = 0;
	
	int x = rand();
	if ( x > 400 ) {
		x = (int) x % 400;
	}
	
	setX(x);
	setY(480+(rand()%900));
	
	int velH = (rand()%12)+18;
	
	if (x<100) {
		velHorizontal = velH;
	} else {
		if (rand()%2==0) {
			velHorizontal = -velH;
		} else {
			velHorizontal = velH;
		}
	}
	velVertical = -((rand()%10)+25);
}

void Balon::dibujar (SDL_Surface *superficie) {

	SDL_Rect dest;
	dest.x = (int)posX;
	dest.y = (int)posY;
	
	if (ciclos%4==0) {
		selSiguienteFrame();
	}

	SDL_SetAlpha(sprite[actualFrame].img, SDL_SRCALPHA, alpha);
	SDL_BlitSurface (sprite[actualFrame].img, NULL, superficie, &dest);

}

/*****************************
**
** CLASE JUGADOR
**
******************************/

Jugador::Jugador(int nf)  : Elemento (nf) {
	estado = 1;
	mostrar = 1;
	mostrarPoder = 1;
	mostrarFlare = 0;
	invisible = 0;
	swEmpezarMovimientoLado = 1;
	ciclosSegundoMovimiento = 0;
	actualizarDibujo = 0;
	disparosContinuos = 0;
	esperaDisparosContinuos = 0;
	pulsadoPoder = 0;
	pulsadoSelPoder = 0;
	tipoMovimiento = MOVIMIENTO_SIN;
	
	velocidad = 8;
	desplDerecha = false;
	desplIzquierda = false;
	
	teclaArriba = SDLK_UP;
	teclaAbajo = SDLK_DOWN;
	teclaIzquierda = SDLK_LEFT;
	teclaDerecha = SDLK_RIGHT;
	
	teclaDisparo_P1 = SDLK_LCTRL;
	teclaDisparo_P2 = SDLK_RCTRL;
	teclaUsarPoder_P1 = SDLK_LALT;
	teclaUsarPoder_P2 = SDLK_RALT;
	
	teclaPoder1 = SDLK_1;
	teclaPoder2 = SDLK_2;
	teclaPoder3 = SDLK_3;
	teclaPoder4 = SDLK_4;
}

void Jugador::cargarSprites(SDL_Surface *screen) {
	char tmpch[127];
	
	// cargamos los diparos de los enemigos
	disp_jugador = new DisparoJugador();
	
	Frame frJugador0, frJugador1, frJugador2, frJugador3, frJugador4, frJugador5, frJugador6;
	Frame frBarrera0, frBarrera1, frBarrera2, frClonIzq, frClonDer, frSombra;
	
	sprintf (tmpch, "%s/tirititran.tim", DATA_DIR);
	frJugador0.loadFromRAR(tmpch, "tirititran0.timp");
	addFrame(frJugador0);
	
	frJugador1.loadFromRAR(tmpch, "tirititran1.timp");
	addFrame(frJugador1);
	
	frJugador2.loadFromRAR(tmpch, "tirititran2.timp");
	addFrame(frJugador2);
	
	frJugador3.loadFromRAR(tmpch, "tirititran3.timp");
	addFrame(frJugador3);
	
	frJugador4.loadFromRAR(tmpch, "tirititran4.timp");
	addFrame(frJugador4);
	
	frJugador5.loadFromRAR(tmpch, "tirititran5.timp");
	addFrame(frJugador5);
	
	frJugador6.loadFromRAR(tmpch, "tirititranexp_0.timp");
	addFrame(frJugador6);

	frJugador3.loadFromRAR(tmpch, "tirititranexp_1.timp");
	addFrame(frJugador3);
	
	sombra = new Elemento();
	frSombra.loadFromRAR(tmpch, "sombra0.timp");
	sombra->addFrame(frSombra);
	
	barrera = new Elemento(3);
	frBarrera0.loadFromRAR(tmpch, "barrera0.timp");
	barrera->addFrame(frBarrera0);
	frBarrera1.loadFromRAR(tmpch, "barrera1.timp");
	barrera->addFrame(frBarrera1);
	frBarrera2.loadFromRAR(tmpch, "barrera2.timp");
	barrera->addFrame(frBarrera2);
	
	clon_der = new Elemento();
	frClonDer.loadFromRAR(tmpch, "clon.timp");
	clon_der->addFrame(frClonDer);
	clon_der->setCortinilla(10);
	
	clon_izq = new Elemento();
	frClonIzq.loadFromRAR(tmpch, "clon.timp");
	clon_izq->addFrame(frClonIzq);
	clon_izq->setCortinilla(10);
	
	// cargar flare	
	Frame frFlare0, frFlare1, frFlare2, frFlare3;
	Frame frFlare4, frFlare5, frFlare6, frFlare7;
	Frame frFlare8, frFlare9, frFlare10, frFlare11;
	Frame frFlare12, frFlare13, frFlare14, frFlare15;
	Frame frFlare16, frFlare17, frFlare18, frFlare19;

	frFlare0.loadFromRAR(tmpch, "flare0.timp");
	frFlare1.loadFromRAR(tmpch, "flare1.timp");
	frFlare2.loadFromRAR(tmpch, "flare2.timp");
	frFlare3.loadFromRAR(tmpch, "flare3.timp");
	frFlare4.loadFromRAR(tmpch, "flare4.timp");
	frFlare5.loadFromRAR(tmpch, "flare5.timp");
	frFlare6.loadFromRAR(tmpch, "flare6.timp");
	frFlare7.loadFromRAR(tmpch, "flare7.timp");
	frFlare8.loadFromRAR(tmpch, "flare8.timp");
	frFlare9.loadFromRAR(tmpch, "flare9.timp");
	frFlare10.loadFromRAR(tmpch, "flare10.timp");
	frFlare11.loadFromRAR(tmpch, "flare11.timp");
	frFlare12.loadFromRAR(tmpch, "flare12.timp");
	frFlare13.loadFromRAR(tmpch, "flare13.timp");
	frFlare14.loadFromRAR(tmpch, "flare14.timp");
	frFlare15.loadFromRAR(tmpch, "flare15.timp");
	frFlare16.loadFromRAR(tmpch, "flare16.timp");
	frFlare17.loadFromRAR(tmpch, "flare17.timp");
	frFlare18.loadFromRAR(tmpch, "flare18.timp");
	frFlare19.loadFromRAR(tmpch, "flare19.timp");
	
	flare = new Elemento(20);
	flare->addFrame(frFlare0);
	flare->addFrame(frFlare1);
	flare->addFrame(frFlare2);
	flare->addFrame(frFlare3);
	flare->addFrame(frFlare4);
	flare->addFrame(frFlare5);
	flare->addFrame(frFlare6);
	flare->addFrame(frFlare7);
	flare->addFrame(frFlare8);
	flare->addFrame(frFlare9);
	flare->addFrame(frFlare10);
	flare->addFrame(frFlare11);
	flare->addFrame(frFlare12);
	flare->addFrame(frFlare13);
	flare->addFrame(frFlare14);
	flare->addFrame(frFlare15);
	flare->addFrame(frFlare16);
	flare->addFrame(frFlare17);
	flare->addFrame(frFlare18);
	flare->addFrame(frFlare19);
	
	
}

void Jugador::setEstado (int est) { 
	estado = est;
	switch (est)
	{
		case 1:
			posX = 260 - (getAncho()/2);
			posY = 480 + getAlto();
			break;
	}

}

void Jugador::crearDisparo(Juego *juego, Audio *sonido) { 
	disp_jugador->crear(sonido, this, juego); 
}

bool Jugador::comprobarTeclas(Juego *juego, Joystick *joystick, Audio *sonido, bool compMovimiento, bool compDisparo, bool compPoderes) {

	if (juego->getNivel()!=0) {
		keys = SDL_GetKeyState (NULL);
		
		// cosultamos el teclado
		//keys = SDL_GetKeyState (NULL);
	
		if (compMovimiento) {
			SDL_JoystickUpdate();
		
			if (SDL_NumJoysticks() >= 1) {
				joyx = SDL_JoystickGetAxis (joystick->joystick, 0);
				joyy = SDL_JoystickGetAxis (joystick->joystick, 1);
			} else {
				joyx = 0;
				joyy = 0;
			}
		
			if (keys[teclaArriba] || joyy < -3200) {
				moverArriba();
			}
			
			if (keys[teclaAbajo] || joyy > 3200) {
				moverAbajo();
			}
					
			if (keys[teclaIzquierda] || joyx < -3200) {
				moverIzquierda();
			}
					
			if (keys[teclaDerecha] || joyx > 3200) {
				moverDerecha();
			}
		}
		
		if (compDisparo) {
			if  ( ( keys[teclaDisparo_P1] ) || ( keys[teclaDisparo_P2] ) || (joystick->comprobarBotonJoystick(JOY_DISPARO)) ) {
				if ( esperaDisparosContinuos <= 0) {
					if ( disparosContinuos < RAFAGADISPAROS) {
						//crearDisparo();
						disp_jugador->crear(sonido, this, juego);
						disparosContinuos++;
						esperaDisparosContinuos = ESPERADISPAROS;
					}
				}
				//pulsadoSelPoder = 1;
				esperaDisparosContinuos--;
			} else {
				//pulsadoSelPoder = 0;
				disparosContinuos = 0;
				esperaDisparosContinuos = 0;
			}
			
			if ( ( ( keys[teclaUsarPoder_P1] ) || ( keys[teclaUsarPoder_P2] ) || (joystick->comprobarBotonJoystick(JOY_USAR_PODER)) ) && ( juego->getPoderes() > 0 )   && ( !juego->getPoderActivo() ) ) {
				setMostrarFlare(1);
				if (pulsadoPoder == 0) {
					/*
					switch (juego->getPoderSel()) {
						case 1:
							sonido->playSonido(SND_FRASE_ESCUDO);
							juego->setPoderActivo(1);
						break;
						case 2:
							estela_bomba->setActivo(1);
							cicloEstela = 0;
							eliminarEnemigosPantalla();
							sonido->playSonido(SND_BOMBA);
						break;
						case 3:
							sonido->playSonido(SND_FRASE_CLONACION);
							juego->setPoderActivo(1);
						break;
						case 4:
							sonido->playSonido(SND_ESE_CADI);
							cicloEstela = 0;
							disp_ene->iniciar();
							for (int i=0; i<MAX_BALON; i++) {
								disp_jugador->balon[i]->iniciar();
								disp_jugador->balon[i]->setActivo(1);
							}
							EnPoderCadista = true;
						break;
					}*/
					juego->removePoderes();
					pulsadoPoder = 1;
					return true;
				}
			} else {
				pulsadoPoder = 0;
			}
	
			
		}
		
		if ( compPoderes ) {
			
			if ( ( keys[teclaPoder1] ) && ( juego->getPoderes() > 0 ) && ( !juego->getPoderActivo() ) ){
				juego->setPoderSel(1);
				sonido->playSonido(SND_BEEP1);
			}
			if ( ( keys[teclaPoder2] ) && ( juego->getPoderes() > 0 ) && ( !juego->getPoderActivo() ) ) {
				juego->setPoderSel(2);
				sonido->playSonido(SND_BEEP1);
			}
			if ( ( keys[teclaPoder3] ) && ( juego->getPoderes() > 0 ) && ( !juego->getPoderActivo() ) ) {
				juego->setPoderSel(3);
				sonido->playSonido(SND_BEEP1);
			}
			if ( ( keys[teclaPoder4] ) && ( juego->getPoderes() > 0 ) && ( !juego->getPoderActivo() ) ) {
				juego->setPoderSel(4);
				sonido->playSonido(SND_BEEP1);
			}	
			
			if ( ( keys[SDLK_x] ) && ( juego->getPoderes() > 0 ) && ( !juego->getPoderActivo() ) ) {
				if (pulsadoSelPoder == 0) {
					sonido->playSonido(SND_BEEP1);
					juego->addPoderSel();
					pulsadoSelPoder = 1;
				} 
			} else if ( ( keys[SDLK_z] ) && ( juego->getPoderes() > 0 ) && ( !juego->getPoderActivo() ) ) {
				if (pulsadoSelPoder == 0) {
					sonido->playSonido(SND_BEEP1);
					juego->removePoderSel();
					pulsadoSelPoder = 1;
				} 
			} else {
				pulsadoSelPoder = 0;
			}
			
	
		}
	}
	return false;
}


void Jugador::moverJugador (Juego *juego) {
	int fr;
	
	if (tipoMovimiento == MOVIMIENTO_ESTABLECERSE_DESDE_DER) {
		actualizarDibujo = 1;
		if (actualFrame == 2) {
			tipoMovimiento = MOVIMIENTO_SIN;
		} else {
			selFrame(2);
		}
	} else if (tipoMovimiento == MOVIMIENTO_ESTABLECERSE_DESDE_IZQ) {
		actualizarDibujo = 1;
		if (actualFrame == 4) {
			tipoMovimiento = MOVIMIENTO_SIN;
		} else {
			selFrame(4);
		}
	} else {

		if (desplIzquierda) {//(keys[SDLK_LEFT] || joyx < -10) {
			if (tipoMovimiento == MOVIMIENTO_DER) {
				swEmpezarMovimientoLado = 1;
			}
			tipoMovimiento = MOVIMIENTO_IZQ;
			if (swEmpezarMovimientoLado == 1) {
				ciclosSegundoMovimiento = 10;
				swEmpezarMovimientoLado = 0;
			} else {
				ciclosSegundoMovimiento--;
			}
			if (ciclosSegundoMovimiento<=0) {
				fr = 5;
			} else {
				fr = 4;
			}
			selFrame(fr);
			desplIzquierda = false;
		} else	if (desplDerecha || joyx > 10) { //else	if (keys[SDLK_RIGHT] || joyx > 10) {
			if (tipoMovimiento == MOVIMIENTO_IZQ) {
				swEmpezarMovimientoLado = 1;
			}
			tipoMovimiento = MOVIMIENTO_DER;
			if (swEmpezarMovimientoLado == 1) {
				ciclosSegundoMovimiento = 10;
				swEmpezarMovimientoLado = 0;
			} else {
				ciclosSegundoMovimiento--;
			}
			if (ciclosSegundoMovimiento<=0) {
				fr = 3;
			} else {
				fr = 2;
			}
			selFrame(fr);
			desplDerecha = false;
		} else {
			swEmpezarMovimientoLado = 1;
			if (tipoMovimiento == MOVIMIENTO_IZQ) {
				tipoMovimiento = MOVIMIENTO_ESTABLECERSE_DESDE_IZQ;
			} else if (tipoMovimiento == MOVIMIENTO_DER) {
				tipoMovimiento = MOVIMIENTO_ESTABLECERSE_DESDE_DER;
			} else {
				tipoMovimiento = MOVIMIENTO_SIN;
				if ( actualFrame == 0 ) {
					fr = 1;
				} else {
					fr = 0;
				}
				if ( ( juego->getCiclos() % 20 == 0 ) || ( actualizarDibujo == 1) ) {
					actualizarDibujo = 0;
					selFrame(fr);
				}
			}
		}
	} 
	
}

void Jugador::morir (Audio *son) {
	if (getEstado()!=0) {
		switch (rand()%12) {
			case 0:		
				son->playSonido (SND_MUERTE0);
			break;
			case 1:		
				son->playSonido (SND_MUERTE1);
			break;
			case 2:		
				son->playSonido (SND_MUERTE2);
			break;
			case 3:		
				son->playSonido (SND_MUERTE3);
			break;
			case 4:		
				son->playSonido (SND_MUERTE4);
			break;
			case 5:		
				son->playSonido (SND_MUERTE5);
			break;
			case 6:		
				son->playSonido (SND_MUERTE6);
			break;
			case 7:		
				son->playSonido (SND_MUERTE7);
			break;
			case 8:		
				son->playSonido (SND_MUERTE8);
			break;
			case 9:		
				son->playSonido (SND_MUERTE9);
			break;
			case 10:		
				son->playSonido (SND_MUERTE10);
			break;
			case 11:		
				son->playSonido (SND_MUERTE11);
			break;
		}
	}
	setEstado(0);
}

void Jugador::dibujar (SDL_Surface *superficie, Juego *juego, Joystick *joystick, Audio *son) {
	int fr, m;
	SDL_Rect dest;
	dest.x = (int)posX;
	dest.y = (int)posY;
	SDL_Rect dest_sombra;
	
	dest_sombra.x = (int)posX + 50;
	dest_sombra.y = (int)posY + 90;
	
	
	switch(estado) {
	case  2:   // vivo
		mostrar = 1;
		moverJugador (juego);
	break;
	case 1:  // naciendo
		moverJugador (juego);
		if ( mostrar ) {
			m = 0;
		} else {
			m = 1;
		}
		if ( juego->getCiclos() % 3 == 0 ) {
			mostrar = m;
		}
		clon_der->setCortinilla(10);
		clon_izq->setCortinilla(10);
		mostrarPoder = 1;
	break;
	case 0:  //muriendo
		mostrar = 1;
		if (actualFrame<6) { // en el caso que muero moviendo hacia izq o der establecer posicion
			selFrame(6);
		}
		if (juego->getCiclos()%4==0) {
			if (actualFrame<7)
				actualFrame++;	
		}
		addY(15);
		if (fueraLimitesDePantalla())
			estado = 3;
	break;
	default:
		mostrar = 1;
		selFrame(0);
	break;
	}
	
	
	if ( mostrar ) {
		SDL_BlitSurface (sombra->sprite[0].img, NULL, superficie, &dest_sombra);
		SDL_BlitSurface (sprite[actualFrame].img, NULL, superficie, &dest);

		if (juego->getPoderActivo()) {
			if (juego->getTiempoPoder() < MAXTIEMPO_PODER) {
				juego->addTiempoPoder();
				if (juego->getTiempoPoder() == MAXTIEMPO_PODER -80) {
					switch (rand()%3) {
						case 0:
							son->playSonido(SND_DESACTIVAR_PODER0);
						break;
						case 1:
							son->playSonido(SND_DESACTIVAR_PODER1);
						break;
						case 2:
							son->playSonido(SND_DESACTIVAR_PODER2);
						break;
					}
				}
				if (juego->getTiempoPoder() > MAXTIEMPO_PODER -80) {
					if ( mostrarPoder ) {
						m = 0;
					} else {
						m = 1;
					}
					if ( juego->getCiclos() % 3 == 0 ) {
						mostrarPoder = m;
					}
				}
				switch (juego->getPoderSel()) {
					case 1:
						if (juego->getCiclos()%4 == 0) {	
							if (barrera->getActualFrame() == 2) {
								fr = 0;
							} else if (barrera->getActualFrame() == 1) {
								fr = 2;	
							} else {
								fr = 1;	
							}
							barrera->selFrame(fr);
						}
						barrera->setX(getCentroX() - barrera->getAncho()/2);
						barrera->setY(getCentroY() - barrera->getAlto()/2);
						if ( (mostrarPoder == 1) && (mostrarFlare==0) ) {
							barrera->dibujar(superficie);
						}
					break;
					case 3:
						clon_der->setX(getCentroX()-40-getAncho()/2);
						clon_der->setY(getCentroY()+15);
						clon_izq->setX(getCentroX()+40);//-getAncho()/2
						clon_izq->setY(getCentroY()+15);
						if (mostrarPoder == 1) {
							clon_der->dibujar(superficie);
							clon_izq->dibujar(superficie);
						}
					break;
				}
			} else { // termina el poder
				juego->setTiempoPoder(0);
				juego->setPoderActivo(0);
				mostrarPoder = 1;
				setFantasma(0);
				clon_der->setCortinilla(10);
				clon_izq->setCortinilla(10);
			}
			if (mostrarFlare==1) {
				flare->setX(getCentroX()-flare->getAncho()/2-2);
				flare->setY(getCentroY()-flare->getAlto()/2);
				flare->dibujar(superficie);
				//if (juego->getCiclos()%2 == 0) {
					flare->selSiguienteFrame();
				//}
				if (flare->getFrames()-1 <= flare->getActualFrame() ) {
					flare->selFrame(0);
					mostrarFlare = 0;
				}
			}
		}
	}
}

void Jugador::unLoad() {
	int i;
	for (i=0; i<nFrames-1; i++) {
		sprite[i].unLoad();
	}
	sombra->unLoad();
	barrera->unLoad();
	clon_izq->unLoad();
	clon_der->unLoad();
}

/*****************************
**
** CLASE ENEMIGO
**
******************************/

Enemigo::Enemigo()  : Elemento () {
	ciclos=0;
	numMovimientos = 0;
	movimientoActual = -1;
	numDisparos = 0;
	disparoActual = -1;
	tipoRegalo = AG_NONE;
}

Enemigo::Enemigo(int nf)  : Elemento (nf) {
	ciclos=0;
	numMovimientos = 0;
	movimientoActual = -1;
	numDisparos = 0;
	disparoActual = -1;
	tipoRegalo = AG_NONE;
}

void Enemigo::addDisparo(int cic, int tipoDisparo, int movDisparo, int nDisparos, int intervalo) {
	int i;
	if ( numDisparos < NUM_DISPAROS ) {
		for ( i=0; i < nDisparos; i++ ) {
			disparoActual++;
			disparos[disparoActual][0] = cic + (intervalo*i);
			disparos[disparoActual][1] = tipoDisparo;
			disparos[disparoActual][2] = movDisparo;
			numDisparos++;
		}
	}
	
}

bool Enemigo::comprobarDisparo() {
	int i;
	for ( i=0; i<numDisparos; i++ ) {
		if ( ciclos == disparos[i][0] ) {
			disparoActual = i;
			return 1;
		}
	}
	
	return 0;
}

/*****************************
**
** CLASE GENERALNIVEL1
**
******************************/

GeneralNivel1::GeneralNivel1()  : Enemigo () {
}

GeneralNivel1::GeneralNivel1(int nf)  : Enemigo (nf) {
}

int GeneralNivel1::direccionMovimiento (int x, int y) {
	if ( ( x > 0) && ( y > 0) ) {
		return	14;
	} else if ( ( x > 0) && ( y == 0) ) {
		return	0;
	} else if ( ( x > 0) && ( y < 0) ) {
		return	2;
	} else if ( ( x < 0) && ( y > 0) ) {
		return	10;
	} else if ( ( x < 0) && ( y == 0) ) {
		return	8;
	} else if ( ( x < 0) && ( y < 0) ) {
		return	6;
	} else if ( ( x == 0) && ( y > 0) ) {
		return	12;
	} else if ( ( x == 0) && ( y == 0) ) {
		return	16;
	} else if ( ( x == 0) && ( y < 0) ) {
		return	4;
	}
	return 0;
}

int GeneralNivel1::direccionCircular (int mov1, int mov2) {
	int posicionesAdelante = 0, posicionesAtras = 0;
	// numero de movimientos en SCAR
	int auxmov1 = mov1;
	while ( auxmov1 != mov2 ) {
		posicionesAdelante++;
		auxmov1++;
		if ( auxmov1 == 16 ) { auxmov1 = 0;	}
	}
	// numero de movimientos en SAR
	auxmov1 = mov1;
	while ( auxmov1 != mov2 ) {
		posicionesAtras++;
		auxmov1--;
		if ( auxmov1 == -1 ) { auxmov1 = 15;	}
	}
	if ( posicionesAdelante > posicionesAtras ) {
		return 0;
	} else {
		return 1;
	}
}

void GeneralNivel1::addMovimiento (int tipoMov, int c, int dx, int dy, int nFrame) {
	int cic, auxX, auxY;
	int dirActual, dirAnterior;
	if (c == -1) {
		cic = 9999;
	} else {
		if ( numMovimientos > 0 ) {
			cic = movimientos[numMovimientos-1][1] + c;
		} else {
			cic = c;
		}
	}
	if (numMovimientos < NUM_MOVIMIENTOS ) {
		dirActual =direccionMovimiento (dx, dy);
		if ( ( nFrame == -1 ) && (tipoMov != MOV_MIRANDOJUGADOR) && (tipoMov != MOV_PARADO) && (tipoMov != MOV_IMPULSAR) && (tipoMov != MOV_FRENAR)) {
			// añadir todos los movimientos intermedios con respecto al movimiento anterior
			if ( numMovimientos > 0 ) {
				dirAnterior =movimientos[numMovimientos-1][4];
				if ( dx == 0 ) {
					auxY = dy;
					auxX = movimientos[numMovimientos-1][2]/2;
				} else {
					auxY = movimientos[numMovimientos-1][3]/2;
					auxX = dx;
				}
				if ( direccionCircular ( dirAnterior, dirActual ) == 1 ) { // movimiento contrario agujas del reloj
					while ( dirActual != dirAnterior ) {
						dirAnterior++;
						if ( dirAnterior == 16 ) { dirAnterior = 0;	}
						addMovimiento (0, 3, auxX, auxY, dirAnterior);
					}
				} else { // movimiento de las agujas del reloj
						while ( dirActual != dirAnterior ) {
						dirAnterior--;
						if ( dirAnterior == -1 ) {	dirAnterior = 15; }
						addMovimiento (0, 3, auxX, auxY, dirAnterior);
					}
				}
			}
		}
		// frenando
		if ( ( tipoMov == 2 )  && ( numMovimientos > 0) ){
			addMovimiento(MOV_FRENAR, 10, movimientos[numMovimientos-1][2], movimientos[numMovimientos-1][3], -1);
			cic += 5;
		}
		
		movimientos[numMovimientos][0] = tipoMov;
		movimientos[numMovimientos][1] = cic;
		movimientos[numMovimientos][2] = dx;
		movimientos[numMovimientos][3] = dy;
		if ( nFrame == -1 ) {
			movimientos[numMovimientos][4] = direccionMovimiento (dx, dy);
		}  else {
			movimientos[numMovimientos][4] = nFrame;
		}
		
		numMovimientos++;
		
	}
}

void GeneralNivel1::mover (int jugadorX) {
	int c1;
	int c2;
	ciclos++;
	if (numMovimientos > 0) { // tiene movimientos
		// comprobar en que movimiento se encuentra el avión según el ciclo recorrido
		for (int i=0; i < numMovimientos; i++ ) {
			if ( i > 0 ) {
				c1 = movimientos[i-1][1];
			} else {
				c1 = 0;
			}
			c2 = movimientos[i][1];

			if ( ( ciclos > c1 ) && ( ciclos < c2 ) ) {
				movimientoActual = i;
			}				
		}
		// movimiento en relacion a "dx" y "dy"
		if ( movimientos[movimientoActual][0] == MOV_NORMAL ) {
			addX( movimientos[movimientoActual][2] );
			addY( movimientos[movimientoActual][3] );
			selFrame(movimientos[movimientoActual][4]);
		}
			
		// movimiento kamikaze
		if ( movimientos[movimientoActual][0] == MOV_KAMIKAZE ) {
			if ( jugadorX < posX + (getAncho()/2) - 25 ) {
				addX( -3 );
				selFrame(16);
			} else if ( jugadorX > posX + (getAncho()/2) + 25 ) {
				addX( 3 );
				selFrame(17);
			} else {
				selFrame(12);
			}
			addY( 6 );
		}
		
		// movimiento mirando jugador
		if ( movimientos[movimientoActual][0] == MOV_MIRANDOJUGADOR ) {
			addX( movimientos[movimientoActual][2] );
			addY( movimientos[movimientoActual][3] );
			if ( jugadorX < posX + (getAncho()/2) - 90 ) {
				selFrame(10);
			} else if ( jugadorX > posX + (getAncho()/2) + 90 ) {
				selFrame(14);
			} else if ( jugadorX < posX + (getAncho()/2) - 25 ) {
				selFrame(11);
			} else if ( jugadorX > posX + (getAncho()/2) + 25 ) {
				selFrame(13);
			} else {
				selFrame(12);
			}
		}
		
		// movimiento mirando parado
		if ( movimientos[movimientoActual][0] == MOV_PARADO ) {
			if ( jugadorX < posX + (getAncho()/2) - 90 ) {
				selFrame(10);
			} else if ( jugadorX > posX + (getAncho()/2) + 90 ) {
				selFrame(14);
			} else if ( jugadorX < posX + (getAncho()/2) - 25 ) {
				selFrame(11);
			} else if ( jugadorX > posX + (getAncho()/2) + 25 ) {
				selFrame(13);
			} else {
				selFrame(12);
			}
		}
		
		//movimientos de impulso y freno
		int c;
		
		if ( movimientos[movimientoActual][0] == MOV_FRENAR ) {
			c = ciclos-movimientos[movimientoActual-1][1];
			if (c<5) {
				addX( (int)(movimientos[movimientoActual][2] / 2 ) ); 
				addY( (int)(movimientos[movimientoActual][3] / 2 ) ); 
			} else {
				addX( (int)(movimientos[movimientoActual][2] / 3 ) ); 
				addY( (int)(movimientos[movimientoActual][3] / 3 ) ); 
			}
			if ( jugadorX < posX + (getAncho()/2) - 90 ) {
				selFrame(10);
			} else if ( jugadorX > posX + (getAncho()/2) + 90 ) {
				selFrame(14);
			} else if ( jugadorX < posX + (getAncho()/2) - 25 ) {
				selFrame(11);
			} else if ( jugadorX > posX + (getAncho()/2) + 25 ) {
				selFrame(13);
			} else {
				selFrame(12);
			}
		}
		
		if ( movimientos[movimientoActual][0] == MOV_IMPULSAR ) {
			c = ciclos-movimientos[movimientoActual-1][1];
			if (c<5) {
				addX( (int)(movimientos[movimientoActual][2] / 3 ) ); 
				addY( (int)(movimientos[movimientoActual][3] / 3 ) ); 
			} else {
				addX( (int)(movimientos[movimientoActual][2] / 2 ) ); 
				addY( (int)(movimientos[movimientoActual][3] / 2 ) ); 
			}
			if ( jugadorX < posX + (getAncho()/2) - 90 ) {
				selFrame(10);
			} else if ( jugadorX > posX + (getAncho()/2) + 90 ) {
				selFrame(14);
			} else if ( jugadorX < posX + (getAncho()/2) - 25 ) {
				selFrame(11);
			} else if ( jugadorX > posX + (getAncho()/2) + 25 ) {
				selFrame(13);
			} else {
				selFrame(12);
			}
		}
		
	} // final del if
}

void GeneralNivel1::duplicarEnFila(GeneralNivel1* est, int espera, int addX, int addY) {
	activo = 1;
	posX = est->getX() + addX;
	posY = est->getY() + addY;
	ciclos = 0;
	numMovimientos = 0;
	movimientoActual = -1;
	addMovimiento(2, espera, 1, 1, -1);
	for (int i=0; i < est->getNumMovimientos(); i++ ) {
		if ( i == 0 ) {
			addMovimiento(est->getMovimiento_Tipo(i), est->getMovimiento_Ciclos(i), est->getMovimiento_X(i), est->getMovimiento_Y(i), est->getMovimiento_nFrame(i));
		} else {
			addMovimiento(est->getMovimiento_Tipo(i), est->getMovimiento_Ciclos(i) - est->getMovimiento_Ciclos(i-1), est->getMovimiento_X(i), est->getMovimiento_Y(i), est->getMovimiento_nFrame(i));
		}
	}
}

int GeneralNivel1::addImpactosParaMorir (Audio *sonido, Juego *juego, Explosion *exp) { 
	int ipm;
	impactosParaMorir--; 
	ipm = impactosParaMorir;
	if ( impactosParaMorir < 1  ) {
		juego->addPuntos(PUNTOS_GENERAL1);
		juego->setHacerClip(0);		
		setActivo(0);
		iniciar();
		switch (tipoElemento) {
			case GAVIOTA1:
				sonido->playSonido(SND_EXP_PAJAROS);
				exp->crearExplosion(EXP_GAVIOTA1, getCentroX(), getCentroY());				
			break;
			case GAVIOTA2:
				sonido->playSonido(SND_EXP_PAJAROS);
				exp->crearExplosion(EXP_GAVIOTA2, getCentroX(), getCentroY());				
			break;
			case NAVEPEQUENYA:
				sonido->playSonido(SND_EXPLOSION);
				exp->crearExplosion(EXP_NAVEPEQUENYA, getCentroX(), getCentroY());				
			break;
		}
	}
	return ipm;
}

void GeneralNivel1::iniciar () {
	activo = 0;
	ciclos = 0;
	numMovimientos = 0;
	movimientoActual = -1;
	tipoRegalo = AG_NONE;
}

/*****************************
**
** CLASE GENERALNIVEL2
**
******************************/

GeneralNivel2::GeneralNivel2()  : Enemigo () {
	impactosParaMorir = IMPACTOS_NAVEMED;
	impacto = 0;
}

GeneralNivel2::GeneralNivel2(int nf)  : Enemigo (nf) {
	impactosParaMorir = IMPACTOS_NAVEMED;
	impacto = 0;
}

void GeneralNivel2::addMovimiento (int tipoMov, int c, int dx, int dy, int nFrame) {
	int cic;
	
	if (numMovimientos == 0) {
		if ( dy > 0 ) {
			posMovimiento = 0;
		} else if ( dy < 0 ) {
			posMovimiento = 2;
		} else {
			if (dx < 0) {
				posMovimiento = 1;
			} else {
				posMovimiento = 3;
			}
		}
	}
	
	if (numMovimientos < NUM_MOVIMIENTOS ) {
		if ( ( tipoMov == 2 )  && ( numMovimientos > 0) ){// frenando
			addMovimiento(MOV_FRENAR, 10, movimientos[numMovimientos-1][2], movimientos[numMovimientos-1][3], -1);
		}
		// impulsando
		if ( ( tipoMov == 0 ) && (movimientos[numMovimientos-1][0] == 2) && ( numMovimientos > 0) && ( nFrame != 0) ) {
			addMovimiento(MOV_IMPULSAR, 10, dx, dy, -1);
		}
	}
	
	if (c == -1) {
		cic = 9999;
	} else {
		if ( numMovimientos > 0 ) {
			cic = movimientos[numMovimientos-1][1] + c;
		} else {
			cic = c;
		}
	}
	
	movimientos[numMovimientos][0] = tipoMov;
	movimientos[numMovimientos][1] = cic;
	movimientos[numMovimientos][2] = dx;
	movimientos[numMovimientos][3] = dy;
	movimientos[numMovimientos][4] = nFrame;
	numMovimientos++;

}

void GeneralNivel2::mover (int jugadorX) {
	int c1;
	int c2;
	ciclos++;
	if (numMovimientos > 0) { // tiene movimientos
		// comprobar en que movimiento se encuentra el avión según el ciclo recorrido
		for (int i=0; i < numMovimientos; i++ ) {
			if ( i > 0 ) {
				c1 = movimientos[i-1][1];
			} else {
				c1 = 0;
			}
			c2 = movimientos[i][1];

			if ( ( ciclos > c1 ) && ( ciclos < c2 ) ) {
				movimientoActual = i;
			}				
		}
		// movimiento en relacion a "dx" y "dy"
		if ( movimientos[movimientoActual][0] == 0 ) {
			addX( movimientos[movimientoActual][2] );
			addY( movimientos[movimientoActual][3] );
		}
		
		int c;
		
		if ( movimientos[movimientoActual][0] == MOV_FRENAR ) {
			c = ciclos-movimientos[movimientoActual-1][1];
			if (c<5) {
				addX( (int)(movimientos[movimientoActual][2] / 2 ) ); 
				addY( (int)(movimientos[movimientoActual][3] / 2 ) ); 
			} else {
				addX( (int)(movimientos[movimientoActual][2] / 3 ) ); 
				addY( (int)(movimientos[movimientoActual][3] / 3 ) ); 
			}
		}
		
		if ( movimientos[movimientoActual][0] == MOV_IMPULSAR ) {
			c = ciclos-movimientos[movimientoActual-1][1];
			if (c<5) {
				addX( (int)(movimientos[movimientoActual][2] / 3 ) ); 
				addY( (int)(movimientos[movimientoActual][3] / 3 ) ); 
			} else {
				addX( (int)(movimientos[movimientoActual][2] / 2 ) ); 
				addY( (int)(movimientos[movimientoActual][3] / 2 ) ); 
			}
		}

					
	} // final del if
}

void GeneralNivel2::dibujar (SDL_Surface *superficie) {
	SDL_Rect dest;
	int fr;
	dest.x = (int)posX;
	dest.y = (int)posY;
	if (impacto == 1) {
		actualFrame = 2 + ( posMovimiento * 3);
		impacto = 0;
	} else {
		if ( actualFrame == posMovimiento * 3 ) {
			fr = 1 + ( posMovimiento * 3);
		} else {
			fr = ( posMovimiento * 3);
		}
		if (ciclos%3 == 0) {
			selFrame(fr);
		}
	}
	SDL_BlitSurface (sprite[actualFrame].img, NULL, superficie, &dest);
}

int GeneralNivel2::addImpactosParaMorir (Audio *sonido, Juego *juego, Explosion *exp) { 
	int ipm;
	impactosParaMorir--; 
	ipm = impactosParaMorir;
	setImpacto(1);
	if ( impactosParaMorir < 1  ) {
		sonido->playSonido(SND_EXPLOSION);
		juego->addPuntos(PUNTOS_GENERAL2);
		juego->setHacerClip(0);		
		setActivo(0);
		iniciar();
		exp->crearExplosion(EXP_NAVEMED, getCentroX(), getCentroY());				
	}
	return ipm;
}

void GeneralNivel2::iniciar () {
	activo = 0;
	ciclos = 0;
	numMovimientos = 0;
	movimientoActual = -1;
	numDisparos = 0;
	disparoActual = -1;
	setImpactosParaMorir(IMPACTOS_NAVEMED);
	impacto = 0;
	tipoRegalo = AG_NONE;
}

/*****************************
**
** CLASE GENERALNIVEL3
**
******************************/

GeneralNivel3::GeneralNivel3()  : Enemigo () {
	impactosParaMorir = IMPACTOS_NAVEGR;
	impacto = 0;
	sombra = new Elemento();
	fuego0 = new Elemento(2);
	fuego1 = new Elemento(2);
}

GeneralNivel3::GeneralNivel3(int nf)  : Enemigo (nf) {
	impactosParaMorir = IMPACTOS_NAVEGR;
	impacto = 0;
	sombra = new Elemento();
	fuego0 = new Elemento(2);
	fuego1 = new Elemento(2);
}

void GeneralNivel3::addMovimiento (int tipoMov, int c, int dx, int dy, int nFrame) {
	int cic;
	
	if (numMovimientos < NUM_MOVIMIENTOS ) {
		if ( ( tipoMov == 2 )  && ( numMovimientos > 0) ){ // frenando
			addMovimiento(MOV_FRENAR, 10, movimientos[numMovimientos-1][2], movimientos[numMovimientos-1][3], -1);
		}
		
		// impulsando
		if ( ( tipoMov == 0 ) && (movimientos[numMovimientos-1][0] == 2) && ( numMovimientos > 0) && ( nFrame != 0) ) {
			addMovimiento(MOV_IMPULSAR, 10, dx, dy, -1);
		}
	}
	
	if (c == -1) {
		cic = 9999;
	} else {
		if ( numMovimientos > 0 ) {
			cic = movimientos[numMovimientos-1][1] + c;
		} else {
			cic = c;
		}
	}
	
	movimientos[numMovimientos][0] = tipoMov;
	movimientos[numMovimientos][1] = cic;
	movimientos[numMovimientos][2] = dx;
	movimientos[numMovimientos][3] = dy;
	movimientos[numMovimientos][4] = nFrame;
	numMovimientos++;

}

void GeneralNivel3::mover (int jugadorX) {
	int c1;
	int c2;
	ciclos++;
	if (numMovimientos > 0) { // tiene movimientos
		// comprobar en que movimiento se encuentra el avión según el ciclo recorrido
		for (int i=0; i < numMovimientos; i++ ) {
			if ( i > 0 ) {
				c1 = movimientos[i-1][1];
			} else {
				c1 = 0;
			}
			c2 = movimientos[i][1];

			if ( ( ciclos > c1 ) && ( ciclos < c2 ) ) {
				movimientoActual = i;
			}				
		}
		// movimiento en relacion a "dx" y "dy"
		if ( movimientos[movimientoActual][0] == MOV_NORMAL ) {
			addX( movimientos[movimientoActual][2] );
			addY( movimientos[movimientoActual][3] );
		}
		
		int c;
		
		if ( movimientos[movimientoActual][0] == MOV_FRENAR ) {
			c = ciclos-movimientos[movimientoActual-1][1];
			if (c<5) {
				addX( (int)(movimientos[movimientoActual][2] / 2 ) ); 
				addY( (int)(movimientos[movimientoActual][3] / 2 ) ); 
			} else {
				addX( (int)(movimientos[movimientoActual][2] / 3 ) ); 
				addY( (int)(movimientos[movimientoActual][3] / 3 ) ); 
			}
		}
		
		if ( movimientos[movimientoActual][0] == MOV_IMPULSAR ) {
			c = ciclos-movimientos[movimientoActual-1][1];
			if (c<5) {
				addX( (int)(movimientos[movimientoActual][2] / 3 ) ); 
				addY( (int)(movimientos[movimientoActual][3] / 3 ) ); 
			} else {
				addX( (int)(movimientos[movimientoActual][2] / 2 ) ); 
				addY( (int)(movimientos[movimientoActual][3] / 2 ) ); 
			}
		}
								
	} // final del if
}

int GeneralNivel3::addImpactosParaMorir (Audio *sonido, Juego *juego, Explosion *exp) { 
	int ipm;
	impactosParaMorir--; 
	ipm = impactosParaMorir;
	setImpacto(1);
	if ( impactosParaMorir <= 0  ) {
		sonido->playSonido(SND_EXPLOSION);
		juego->addPuntos(PUNTOS_GENERAL3);
		juego->setHacerClip(0);		
		setActivo(0);
		exp->crearExplosion(EXP_NAVEGRANDE, getCentroX(), getCentroY());				
	}
	return ipm;
}

void GeneralNivel3::iniciar () {
	activo = 0;
	ciclos = 0;
	numMovimientos = 0;
	movimientoActual = -1;
	numDisparos = 0;
	disparoActual = -1;
	setImpactosParaMorir(IMPACTOS_NAVEGR);
	impacto = 0;
	tipoRegalo = AG_NONE;
}

void GeneralNivel3::dibujar (SDL_Surface *superficie) {
	int fr;
	int x1, y1, x2, y2;
	SDL_Rect dest;
	dest.x = (int)posX;
	dest.y = (int)posY;
	SDL_Rect dest_sombra;
	dest_sombra.x = (int)posX + 70;
	dest_sombra.y = (int)posY + 110;

	if (impacto == 1) {
		fr = 2;
		impacto = 0;
	} else {
		if ( actualFrame == 0) {
			fr = 1;
		} else {
			fr = 0;
		}
	}	
	selFrame(fr);

	SDL_BlitSurface (sombra->sprite[0].img, NULL, superficie, &dest_sombra);
	SDL_BlitSurface (sprite[actualFrame].img, NULL, superficie, &dest);
	
	if (tipoElemento == NAVEZEPELIN) {
		x1 = 0;
		y1 = 10;
		x2 = 50;
		y2 = 40;
	} else if (tipoElemento == NAVEGRANDE) {
		x1 = 5;
		y1 = 40;
		x2 = 60;
		y2 = 30;
	} else if (tipoElemento == NAVERASCASIO) {
		x1 = 0;
		y1 = 10;
		x2 = 75;
		y2 = 15;
	}
		
	
	if ( impactosParaMorir < IMPACTOS_NAVEGR/2 +3) {
		fuego0->setX(posX + x1);	
		fuego0->setY(posY + y1);
		if ( fuego0->getActualFrame() == 0 ) {
			fr  = 1;
		} else {
			fr = 0;
		}
		if ( ciclos % 5 == 0 ) {
			fuego0->selFrame(fr);
		}
		fuego0->dibujar(superficie);
	}
	
	if ( impactosParaMorir < IMPACTOS_NAVEGR/2-1) {
		fuego1->setX(posX + x2);
		fuego1->setY(posY + y2);
		if ( fuego1->getActualFrame() == 0 ) {
			fr  = 1;
		} else {
			fr = 0;
		}
		if ( ciclos % 3 == 0 ) {
			fuego1->selFrame(fr);
		}
		fuego1->dibujar(superficie);
	}
	
}

void GeneralNivel3::unLoad() {
	int i;
	for (i=0; i<nFrames-1; i++) {
		sprite[i].unLoad();
	}
	sombra->unLoad();
	fuego0->unLoad();
	fuego1->unLoad();
}


/*****************************
**
** CLASE MARINOS
**
******************************/

Marinos::Marinos()  : Enemigo () {
	impactosParaMorir = 1;
	moverY = 2;
	tiempoEsconderse = -1;
}

Marinos::Marinos(int nf)  : Enemigo (nf) {
	impactosParaMorir = 1;
	moverY = 2;
	tiempoEsconderse = -1;
}

void Marinos::mover() {
	ciclos++;
	addY(moverY);
}

void Marinos::iniciar() {
	activo = 0;
	ciclos = 0;
	moverY = 2;
	numDisparos = 0;
	disparoActual = -1;
	tiempoEsconderse = -1;
	if (getTipoElemento() == PULPO) {
		setImpactosParaMorir(IMPACTOS_PULPO);
	} else {
		setImpactosParaMorir(IMPACTOS_CANGREBOT);
	}
	selFrame(0);
	tipoRegalo = AG_NONE;
}

int Marinos::addImpactosParaMorir (Audio *sonido, Juego *juego, Explosion *exp) { 
	int ipm;
	impactosParaMorir--; 
	ipm = impactosParaMorir;
	//setImpacto(1);
	if ( impactosParaMorir < 1  ) {
		sonido->playSonido(SND_EXPAGUA);
		juego->addPuntos(PUNTOS_MARINOS);
		juego->setHacerClip(0);		
		setActivo(0);
		iniciar();
		if (getTipoElemento() == PULPO) {
			exp->crearExplosion(EXP_PULPO, getCentroX(), getCentroY());	
		} else {
			exp->crearExplosion(EXP_CANGREBOT, getCentroX(), getCentroY());	
		}
	}
	return ipm;
}

void Marinos::dibujar (SDL_Surface *superficie) {
	int fr;
	SDL_Rect dest;
	dest.x = (int)posX;
	dest.y = (int)posY;

	if (actualFrame < 3) {
		setFantasma(1);
	} else {
		setFantasma(0);
	}
	if ( ciclos % 5 == 0 ) {
		fr = actualFrame + 1;
		if ( actualFrame > 5 ) {
			fr = 4;
		}
		selFrame (fr);
	}

	if (tiempoEsconderse>0) {
		tiempoEsconderse--;
	} else if (tiempoEsconderse==0) {
		fr = actualFrame - 1;
		if ( fr < 0 ) {
			setActivo(0);
			iniciar();
			return;
		}
		selFrame (fr);
	}
	
	SDL_SetAlpha(sprite[actualFrame].img, SDL_SRCALPHA, alpha);
	SDL_BlitSurface (sprite[actualFrame].img, NULL, superficie, &dest);

}


/*****************************
**
** CLASE COCHESIMPLE
**
******************************/

CocheSimple::CocheSimple()  : Enemigo () {
	impactosParaMorir = IMPACTOS_COCHESIMPLE;
	impacto = 0;
	maxCoches = 1;
	cocheActual = 0;
	posMovimiento = 0;
	selFrame(0);
}

CocheSimple::CocheSimple(int nf)  : Enemigo (nf) {
	impactosParaMorir = IMPACTOS_COCHESIMPLE;
	impacto = 0;
	maxCoches = nf/2;
	cocheActual = 0;
	posMovimiento = 0;
	selFrame(0);
}

void CocheSimple::selCoche(int tipoCoche) {
	cocheActual = tipoCoche;
	selFrame(cocheActual*2);
}


void CocheSimple::addMovimiento (int tipoMov, int c, int dx, int dy, int nFrame) {
	int cic;
	//int auxX1, auxY1, auxX2, auxY2;
	
	if (numMovimientos == 0) {
		if ( dy > 0 ) {
			posMovimiento = 0;
		} else {
			posMovimiento = 1;
			actualFrame++;
		}
	}
	
	if (c == -1) {
		cic = 9999;
	} else {
		if ( numMovimientos > 0 ) {
			cic = movimientos[numMovimientos-1][1] + c;
		} else {
			cic = c;
		}
	}
	
	movimientos[numMovimientos][0] = tipoMov;
	movimientos[numMovimientos][1] = cic;
	movimientos[numMovimientos][2] = dx;
	movimientos[numMovimientos][3] = dy;
	movimientos[numMovimientos][4] = nFrame;
	numMovimientos++;

}

void CocheSimple::mover (int jugadorX) {
	int c1;
	int c2;
	ciclos++;
	if (numMovimientos > 0) { // tiene movimientos
		// comprobar en que movimiento se encuentra según el ciclo recorrido
		for (int i=0; i < numMovimientos; i++ ) {
			if ( i > 0 ) {
				c1 = movimientos[i-1][1];
			} else {
				c1 = 0;
			}
			c2 = movimientos[i][1];

			if ( ( ciclos > c1 ) && ( ciclos < c2 ) ) {
				movimientoActual = i;
			}				
		}
		// movimiento en relacion a "dx" y "dy"
		if ( movimientos[movimientoActual][0] == 0 ) {
			addX( movimientos[movimientoActual][2] );
			addY( movimientos[movimientoActual][3] );
		}
					
	} // final del if
}


void CocheSimple::iniciar() {
	activo = 0;
	ciclos = 0;
	numDisparos = 0;
	disparoActual = -1;
	setImpactosParaMorir(IMPACTOS_COCHESIMPLE);
	impacto = 0;
	selFrame(0);
	tipoRegalo = AG_NONE;
	numMovimientos = 0;
}

int CocheSimple::addImpactosParaMorir (Audio *sonido, Juego *juego, Explosion *exp) { 
	int ipm;
	impactosParaMorir--; 
	ipm = impactosParaMorir;
	setImpacto(1);
	if ( impactosParaMorir < 1  ) {
		sonido->playSonido(SND_EXPLOSION);
		juego->addPuntos(PUNTOS_COCHESIMPLE);
		juego->setHacerClip(0);		
		setActivo(0);
		if ( (juego->getEnFinal()) || (juego->getMapaParado()) ) {
			exp->crearExplosion(EXP_COCHE_CORTA, getCentroX(), getCentroY());// coordenadas en el centro
		} else {
			exp->crearExplosion(EXP_COCHE, getCentroX(), getCentroY());// coordenadas en el centro
		}
		iniciar();
	}
	return ipm;
}

void CocheSimple::dibujar (SDL_Surface *superficie) {

	SDL_Rect dest;
	dest.x = (int)posX;
	dest.y = (int)posY;

	SDL_SetAlpha(sprite[actualFrame].img, SDL_SRCALPHA, alpha);
	SDL_BlitSurface (sprite[actualFrame].img, NULL, superficie, &dest);

}


/*****************************
**
** CLASE CONCHA
**
******************************/

Concha::Concha()  : Enemigo () {
	impactosParaMorir = IMPACTOS_CONCHA;
	impacto = 0;
	selFrame(0);
	escondido = false;
}

Concha::Concha(int nf)  : Enemigo (nf) {
	impactosParaMorir = IMPACTOS_CONCHA;
	impacto = 0;
	selFrame(0);
	escondido = false;
}


void Concha::addMovimiento (int tipoMov, int c, int dx, int dy, int nFrame) {
	int cic;
	//int auxX1, auxY1, auxX2, auxY2;
	
	
	if (c == -1) {
		cic = 9999;
	} else {
		if ( numMovimientos > 0 ) {
			cic = movimientos[numMovimientos-1][1] + c;
		} else {
			cic = c;
		}
	}
	
	movimientos[numMovimientos][0] = tipoMov;
	movimientos[numMovimientos][1] = cic;
	movimientos[numMovimientos][2] = dx;
	movimientos[numMovimientos][3] = dy;
	movimientos[numMovimientos][4] = nFrame;
	numMovimientos++;

}

void Concha::mover () {
	int c1;
	int c2;
	ciclos++;
	if (numMovimientos > 0) { // tiene movimientos
		// comprobar en que movimiento se encuentra según el ciclo recorrido
		for (int i=0; i < numMovimientos; i++ ) {
			if ( i > 0 ) {
				c1 = movimientos[i-1][1];
			} else {
				c1 = 0;
			}
			c2 = movimientos[i][1];

			if ( ( ciclos > c1 ) && ( ciclos < c2 ) ) {
				movimientoActual = i;
			}				
		}
		// movimiento en relacion a "dx" y "dy"
		if ( movimientos[movimientoActual][0] == 0 ) {
			addX( movimientos[movimientoActual][2] );
			addY( movimientos[movimientoActual][3] );
			escondido = false;
		}
		if ( movimientos[movimientoActual][0] == 2 ) {
			addY( VEL_MAPA );
			escondido = true;
		}
					
	} // final del if
}


void Concha::iniciar() {
	activo = 0;
	ciclos = 0;
	numDisparos = 0;
	disparoActual = -1;
	setImpactosParaMorir(IMPACTOS_CONCHA);
	impacto = 0;
	selFrame(0);
	tipoRegalo = AG_NONE;
	numMovimientos = 0;
	escondido = false;
}

int Concha::addImpactosParaMorir (Audio *sonido, Juego *juego, Explosion *exp) { 
	if (!escondido) {
		int ipm;
		impactosParaMorir--; 
		ipm = impactosParaMorir;
		setImpacto(1);
		if ( impactosParaMorir < 1  ) {
			sonido->playSonido(SND_EXPLOSION);
			juego->addPuntos(PUNTOS_CONCHA);
			juego->setHacerClip(0);		
			setActivo(0);
			if (juego->getMapaParado()) {
				exp->crearExplosion(EXP_COCHE_CORTA, getCentroX(), getCentroY());// coordenadas en el centro
			} else {
				exp->crearExplosion(EXP_COCHE, getCentroX(), getCentroY());// coordenadas en el centro
			}
			iniciar();
		}
		return ipm;
	}
	return 0;
}

void Concha::dibujar (SDL_Surface *superficie) {

	SDL_Rect dest;
	dest.x = (int)posX;
	dest.y = (int)posY;
	
	if (ciclos%4==0) {
		if (escondido) {
			if (actualFrame!=4) {
				actualFrame++;	
			}
		} else {
			if (actualFrame>3)
				selFrame(0);
			selSiguienteFrame_ReboteHasta(3);
		}
	}

	SDL_SetAlpha(sprite[actualFrame].img, SDL_SRCALPHA, alpha);
	SDL_BlitSurface (sprite[actualFrame].img, NULL, superficie, &dest);

}

/*****************************
**
** CLASE CARROZAS
**
******************************/

Carroza::Carroza()  : Enemigo () {
	impactosParaMorir = IMPACTOS_CARROZA;
	impacto = 0;
	maxCarrozas = 1;
	carrozaActual = 0;
	selFrame(0);
	movimientoFrame = 1;
	
	// cargar tractor
	char tmpch[127];
	tractor = new Elemento();
	
	Frame frTractor;
	sprintf (tmpch, "%s/carrozas.tim", DATA_DIR);

	frTractor.loadFromRAR(tmpch, "tractor.timp");
	
	tractor->addFrame(frTractor);
	
	fuego0 = new Elemento(3);
	fuego1 = new Elemento(3);
	humo = new Elemento(3);
	
}

Carroza::Carroza(int nf)  : Enemigo (nf) {
	impactosParaMorir = IMPACTOS_CARROZA;
	impacto = 0;
	maxCarrozas = nf;
	carrozaActual = 0;
	selFrame(0);
	movimientoFrame = 1;
	
	// cargar tractor
	char tmpch[127];
	tractor = new Elemento();
	
	Frame frTractor;
	sprintf (tmpch, "%s/carrozas.tim", DATA_DIR);

	frTractor.loadFromRAR(tmpch, "tractor.timp");
	
	tractor->addFrame(frTractor);
	
	fuego0 = new Elemento(3);
	fuego1 = new Elemento(3);
	humo = new Elemento(3);
	
}

void Carroza::selCarroza(int tipoCarroza) {
	carrozaActual = tipoCarroza;
	selFrame(carrozaActual*4);
}


void Carroza::mover (int jugadorX) {

	ciclos++;
	
	if ( (ciclos>90) && (ciclos<190) ) {
		addY( 1 );
	} else {
		addY( 3 );
	}
}


void Carroza::iniciar() {
	activo = 0;
	ciclos = 0;
	numDisparos = 0;
	disparoActual = -1;
	setImpactosParaMorir(IMPACTOS_CARROZA);
	impacto = 0;
	selFrame(0);
	tipoRegalo = AG_NONE;
	numMovimientos = 0;
	movimientoFrame = 1;
	fantasma = 0;
}

int Carroza::addImpactosParaMorir (Audio *sonido, Juego *juego, Explosion *exp) { 
	int ipm;
	impactosParaMorir--; 
	ipm = impactosParaMorir;
	setImpacto(1);
	if ( impactosParaMorir < 1  ) {
		sonido->playSonido(SND_EXP_JEFES);
		juego->addPuntos(PUNTOS_CARROZA);
		juego->setHacerClip(0);		
		setFantasma(1);
		exp->crearExplosion(EXP_MEDIANA, getCentroX()-40, getCentroY()-20);
		exp->crearExplosion(EXP_NAVEMED, getCentroX()+30, getCentroY()-80);
		exp->crearExplosion(EXP_NAVEMED, getCentroX()-10, getCentroY()-60);
		exp->crearExplosion(EXP_NAVEGRANDE, getCentroX(), getCentroY());
		exp->crearExplosion(EXP_MEDIANA1, getCentroX(), getCentroY()+80);
	}
	return ipm;
}

void Carroza::dibujar (SDL_Surface *superficie) {
	
	if (fueraLimitesDePantalla()) {
		iniciar();
	}

	tractor->setX( posX + (getAncho()/2) - (tractor->getAncho()/2) );
	tractor->setY( posY + getAlto() );
	
	tractor->dibujar(superficie);
	

	SDL_Rect dest;
	dest.x = (int)posX;
	dest.y = (int)posY;
	
	if ( impactosParaMorir < 1  ) {
		actualFrame = (carrozaActual*4)+3;
	} else {
		if (ciclos%8==0) {
			actualFrame += movimientoFrame;
			if ((carrozaActual*3)+carrozaActual == getActualFrame()) {
				movimientoFrame = 1;
			} else if ( (carrozaActual*3)+carrozaActual+2 == getActualFrame()) {
				movimientoFrame = -1;
			}
		}
	}
	
	SDL_SetAlpha(sprite[actualFrame].img, SDL_SRCALPHA, alpha);
	SDL_BlitSurface (sprite[actualFrame].img, NULL, superficie, &dest);
	
	if (impactosParaMorir<25) {
		if (ciclos%3==0) {
			fuego0->selSiguienteFrame();
		}
		if (carrozaActual==DEMONIO) {
			fuego0->setX(posX+65);
			fuego0->setY(posY+5);
		} else {
			fuego0->setX(posX+10);
			fuego0->setY(posY+5);
		}
		fuego0->dibujar(superficie);
	}
	
	if (impactosParaMorir<18) {
		if (ciclos%8==0) {
			humo->selSiguienteFrame();
		}
		if (carrozaActual==DEMONIO) {
			humo->setX(posX+145);
			humo->setY(posY+55);
		} else {
			humo->setX(posX+70);
			humo->setY(posY+55);
		}
		humo->dibujar(superficie);
	}
	
	if (impactosParaMorir<10) {
		if (ciclos%3==0) {
			fuego1->selSiguienteFrame();
		}
		if (carrozaActual==DEMONIO) {
			fuego1->setX(posX+70);
			fuego1->setY(posY+95);
		} else {
			fuego1->setX(posX+60);
			fuego1->setY(posY+95);
		}
		fuego1->dibujar(superficie);
	}

}


/*****************************
**
** CLASE AUTOMARINO
**
******************************/

AutoMarino::AutoMarino()  : Enemigo2Frames () {
}

AutoMarino::AutoMarino(int nf)  : Enemigo2Frames (nf) {
}


/*****************************
**
** CLASE HELICOPTERO
**
******************************/

Helicoptero::Helicoptero()  : Enemigo2Frames () {
}

Helicoptero::Helicoptero(int nf)  : Enemigo2Frames (nf) {
}

/*****************************
**
** CLASE COCHEPOLICIA
**
******************************/

CochePolicia::CochePolicia()  : Enemigo2Frames () {
	impactosParaMorir = IMPACTOS_COCHEPOLICIA;
	setIntervaloFrames(8);
}

CochePolicia::CochePolicia(int nf)  : Enemigo2Frames (nf) {
	impactosParaMorir = IMPACTOS_COCHEPOLICIA;
	setIntervaloFrames(8);
}

int CochePolicia::addImpactosParaMorir (Audio *sonido, Juego *juego, Explosion *exp) { 
	int ipm;
	impactosParaMorir--; 
	ipm = impactosParaMorir;
	if ( impactosParaMorir < 1  ) {
		sonido->playSonido(SND_EXPLOSION);
		juego->addPuntos(PUNTOS_COCHEPOLICIA);
		juego->setHacerClip(0);		
		setActivo(0);
		iniciar();
		exp->crearExplosion(EXP_COCHE, getCentroX(), getCentroY());				
	}
	return ipm;
}

void CochePolicia::addMovimiento (int tipoMov, int c, int dx, int dy, int nFrame) {
	int cic, auxX, auxY;
	int dirActual, dirAnterior;
	if (c == -1) {
		cic = 9999;
	} else {
		if ( numMovimientos > 0 ) {
			cic = movimientos[numMovimientos-1][1] + c;
		} else {
			cic = c;
		}
	}

	if (numMovimientos < NUM_MOVIMIENTOS ) {
		dirActual =direccionMovimiento (dx, dy);
		if ( nFrame == -1 ) {
			// añadir todos los movimientos intermedios con respecto al movimiento anterior
			if ( numMovimientos > 0 ) {
				dirAnterior =movimientos[numMovimientos-1][4];
				if ( dx == 0 ) {
					if ( (dy >= VEL_MAPA) || (dy < 0) ){
						auxY = dy;
					} else {
						auxY = VEL_MAPA;
					}
					auxX = movimientos[numMovimientos-1][2]/2;
				} else {
					if ( (movimientos[numMovimientos-1][3]/2 >= VEL_MAPA ) || (movimientos[numMovimientos-1][3]/2 < 0) ) {
						auxY = movimientos[numMovimientos-1][3]/2;
					} else {
						auxY = VEL_MAPA;
					}
					auxX = dx;
				}
				if ( direccionCircular ( dirAnterior, dirActual ) == 1 ) { // movimiento contrario agujas del reloj
					while ( dirActual != dirAnterior ) {
						dirAnterior++;
						if ( dirAnterior == 16 ) { dirAnterior = 0;	}
						addMovimiento (0, 3, auxX, auxY, dirAnterior);
					}
				} else { // movimiento de las agujas del reloj
						while ( dirActual != dirAnterior ) {
						dirAnterior--;
						if ( dirAnterior == -1 ) {	dirAnterior = 15; }
						addMovimiento (0, 3, auxX, auxY, dirAnterior);
					}
				}
			}
		}
		movimientos[numMovimientos][0] = tipoMov;
		movimientos[numMovimientos][1] = cic;
		movimientos[numMovimientos][2] = dx;
		if ( (dx!=0) && (dy ==0 ) ) {
			movimientos[numMovimientos][3] = VEL_MAPA;
		} else {
			movimientos[numMovimientos][3] = dy;
		}
		if ( nFrame == -1 ) {
			movimientos[numMovimientos][4] = direccionMovimiento (dx, dy);
		}  else {
			movimientos[numMovimientos][4] = nFrame;
		}
		numMovimientos++;
	}
}

void CochePolicia::iniciar () {
	activo = 0;
	ciclos = 0;
	numMovimientos = 0;
	movimientoActual = -1;
	tipoRegalo = AG_NONE;
	impactosParaMorir = IMPACTOS_COCHEPOLICIA;
}

/*****************************
**
** CLASE TAXI
**
******************************/

Taxi::Taxi()  : GeneralNivel1 () {
	impactosParaMorir = IMPACTOS_TAXI;
}

Taxi::Taxi(int nf)  : GeneralNivel1 (nf) {
	impactosParaMorir = IMPACTOS_TAXI;
}

int Taxi::addImpactosParaMorir (Audio *sonido, Juego *juego, Explosion *exp) { 
	int ipm;
	impactosParaMorir--; 
	ipm = impactosParaMorir;
	if ( impactosParaMorir < 1  ) {
		sonido->playSonido(SND_EXPLOSION);
		juego->addPuntos(PUNTOS_TAXI);
		juego->setHacerClip(0);		
		setActivo(0);
		iniciar();
		exp->crearExplosion(EXP_COCHE, getCentroX(), getCentroY());				
	}
	return ipm;
}

void Taxi::addMovimiento (int tipoMov, int c, int dx, int dy, int nFrame) {
	int cic, auxX, auxY;
	int dirActual, dirAnterior;
	if (c == -1) {
		cic = 9999;
	} else {
		if ( numMovimientos > 0 ) {
			cic = movimientos[numMovimientos-1][1] + c;
		} else {
			cic = c;
		}
	}

	if (numMovimientos < NUM_MOVIMIENTOS ) {
		dirActual =direccionMovimiento (dx, dy);
		if ( nFrame == -1 ) {
			// añadir todos los movimientos intermedios con respecto al movimiento anterior
			if ( numMovimientos > 0 ) {
				dirAnterior =movimientos[numMovimientos-1][4];
				if ( dx == 0 ) {
					if ( (dy >= VEL_MAPA) || (dy < 0) ){
						auxY = dy;
					} else {
						auxY = VEL_MAPA;
					}
					auxX = movimientos[numMovimientos-1][2]/2;
				} else {
					if ( (movimientos[numMovimientos-1][3]/2 >= VEL_MAPA ) || (movimientos[numMovimientos-1][3]/2 < 0) ) {
						auxY = movimientos[numMovimientos-1][3]/2;
					} else {
						auxY = VEL_MAPA;
					}
					auxX = dx;
				}
				if ( direccionCircular ( dirAnterior, dirActual ) == 1 ) { // movimiento contrario agujas del reloj
					while ( dirActual != dirAnterior ) {
						dirAnterior++;
						if ( dirAnterior == 16 ) { dirAnterior = 0;	}
						addMovimiento (0, 3, auxX, auxY, dirAnterior);
					}
				} else { // movimiento de las agujas del reloj
						while ( dirActual != dirAnterior ) {
						dirAnterior--;
						if ( dirAnterior == -1 ) {	dirAnterior = 15; }
						addMovimiento (0, 3, auxX, auxY, dirAnterior);
					}
				}
			}
		}
		movimientos[numMovimientos][0] = tipoMov;
		movimientos[numMovimientos][1] = cic;
		movimientos[numMovimientos][2] = dx;
		if ( (dx!=0) && (dy ==0 ) ) {
			movimientos[numMovimientos][3] = VEL_MAPA;
		} else {
			movimientos[numMovimientos][3] = dy;
		}
		if ( nFrame == -1 ) {
			movimientos[numMovimientos][4] = direccionMovimiento (dx, dy);
		}  else {
			movimientos[numMovimientos][4] = nFrame;
		}
		numMovimientos++;
	}
}

void Taxi::iniciar () {
	activo = 0;
	ciclos = 0;
	numMovimientos = 0;
	movimientoActual = -1;
	tipoRegalo = AG_NONE;
	impactosParaMorir = IMPACTOS_TAXI;
}

/*****************************
**
** CLASE ENEMIGO2FRAMES
**
******************************/

Enemigo2Frames::Enemigo2Frames()  : GeneralNivel1 () {
	intervaloFrames = 2;
}

Enemigo2Frames::Enemigo2Frames(int nf)  : GeneralNivel1 (nf) {
	intervaloFrames = 2;
}

int Enemigo2Frames::addImpactosParaMorir (Audio *sonido, Juego *juego, Explosion *exp) { 
	int ipm;
	impactosParaMorir--; 
	ipm = impactosParaMorir;
	if ( impactosParaMorir < 1  ) {
		switch (tipoElemento) {
			case LANCHA:
				sonido->playSonido(SND_EXPAGUA);
			break;
			case HELICOPTERO:
				sonido->playSonido(SND_EXPLOSION);
			break;
		}
		juego->addPuntos(PUNTOS_GENERAL1);
		juego->setHacerClip(0);		
		setActivo(0);
		iniciar();
		switch (tipoElemento) {
			case LANCHA:
				exp->crearExplosion(EXP_LANCHA, getCentroX(), getCentroY());				
			break;
			case HELICOPTERO:
				exp->crearExplosion(EXP_NAVEMED, getCentroX(), getCentroY());
			break;
		}
	}
	return ipm;
}

void Enemigo2Frames::dibujar (SDL_Surface *superficie) {
	SDL_Rect dest;
	dest.x = (int)posX;
	dest.y = (int)posY;
	if (ciclos % (intervaloFrames*2) >= intervaloFrames) {
		actualFrame = actualFrame + 16;
	}
	SDL_SetAlpha(sprite[actualFrame].img, SDL_SRCALPHA, alpha);
	SDL_BlitSurface (sprite[actualFrame].img, NULL, superficie, &dest);
}


/*****************************
**
** CLASE ENEMIGOCONTORRETA
**
******************************/

EnemigoConTorreta::EnemigoConTorreta()  : GeneralNivel1 () {
	posTorreta = -1;
}

EnemigoConTorreta::EnemigoConTorreta(int nf)  : GeneralNivel1 (nf) {
	posTorreta = -1;
}

int EnemigoConTorreta::addImpactosParaMorir (Audio *sonido, Juego *juego, Explosion *exp) { 
	int ipm;
	impactosParaMorir--; 
	ipm = impactosParaMorir;
	if ( impactosParaMorir < 1  ) {
		sonido->playSonido(SND_EXP_GRANDE);
		juego->addPuntos(PUNTOS_TANQUE);
		juego->setHacerClip(0);		
		setActivo(0);
		iniciar();
		switch (tipoElemento) {
			case TANQUE:
				if (juego->getMapaParado()) {
					exp->crearExplosion(EXP_MEDIANA, getCentroX(), getCentroY());// coordenadas en el centro
				} else {
					exp->crearExplosion(EXP_TANQUE, getCentroX(), getCentroY());// coordenadas en el centro
				}
			break;
			case YATE:
				exp->crearExplosion(EXP_YATE, getCentroX(), getCentroY());
			break;
		}
	}
	return ipm;
}

void EnemigoConTorreta::addMovimiento (int tipoMov, int c, int dx, int dy, int nFrame) {
	int cic, auxX, auxY;
	int dirActual, dirAnterior;
	if (c == -1) {
		cic = 9999;
	} else {
		if ( numMovimientos > 0 ) {
			cic = movimientos[numMovimientos-1][1] + c;
		} else {
			cic = c;
		}
	}

	if (numMovimientos < NUM_MOVIMIENTOS ) {
		dirActual =direccionMovimiento (dx, dy);
		if ( ( nFrame == -1 ) && (tipoMov != MOV_PARADO_ENFINAL) ) {
			// añadir todos los movimientos intermedios con respecto al movimiento anterior
			if ( numMovimientos > 0 ) {
				dirAnterior =movimientos[numMovimientos-1][4];
				if ( dx == 0 ) {
					if ( (dy >= VEL_MAPA) || (dy < 0) ){
						auxY = dy;
					} else {
						auxY = 2;
					}
					auxX = movimientos[numMovimientos-1][2]/2;
				} else {
					if ( (movimientos[numMovimientos-1][3]/2 >= VEL_MAPA ) || (movimientos[numMovimientos-1][3]/2 < 0) ) {
						auxY = movimientos[numMovimientos-1][3]/2;
					} else {
						auxY = VEL_MAPA;
					}
					auxX = dx;
				}
				if ( direccionCircular ( dirAnterior, dirActual ) == 1 ) { // movimiento contrario agujas del reloj
					while ( dirActual != dirAnterior ) {
						dirAnterior++;
						if ( dirAnterior == 16 ) { dirAnterior = 0;	}
						addMovimiento (0, 3, auxX, auxY, dirAnterior);
					}
				} else { // movimiento de las agujas del reloj
						while ( dirActual != dirAnterior ) {
						dirAnterior--;
						if ( dirAnterior == -1 ) {	dirAnterior = 15; }
						addMovimiento (0, 3, auxX, auxY, dirAnterior);
					}
				}
			}
		}
		movimientos[numMovimientos][0] = tipoMov;
		movimientos[numMovimientos][1] = cic;
		movimientos[numMovimientos][2] = dx;
		if ( (dx!=0) && (dy ==0 ) ) {
			movimientos[numMovimientos][3] = 0;
		} else {
			movimientos[numMovimientos][3] = dy;
		}
		if ( nFrame == -1 ) {
			movimientos[numMovimientos][4] = direccionMovimiento (dx, dy);
		}  else {
			movimientos[numMovimientos][4] = nFrame;
		}
		numMovimientos++;
	}
}


void EnemigoConTorreta::dibujar (SDL_Surface *superficie, Jugador *j) {
	SDL_Rect dest;
	dest.x = (int)posX;
	dest.y = (int)posY;
	SDL_SetAlpha(sprite[actualFrame].img, SDL_SRCALPHA, alpha);
	SDL_BlitSurface (sprite[actualFrame].img, NULL, superficie, &dest);
	// pintar torreta
	//SDL_SetAlpha(sprite[18].img, SDL_SRCALPHA, alpha);
	dest.x = (int)posX;
	dest.y = (int)posY;
	
	if ( j->getY() > posY+getAlto() ) {
		if ( j->getX() < posX) {
			posTorreta = 5;
		} else if ( j->getX() > posX+getAncho()) {
			posTorreta = 7;
		} else {
			posTorreta = 6;
		}
	} else if ( j->getY() < posY) {
		if ( j->getX() < posX) {
			posTorreta = 3;
		} else if ( j->getX() > posX+getAncho()) {
			posTorreta = 1;
		} else {
			posTorreta = 2;
		}
	} else {
		if ( j->getX() < posX) {
			posTorreta = 4;
		} else if ( j->getX() > posX+getAncho()) {
			posTorreta = 0;
		} else {
			posTorreta = 6;
		}
	}

	SDL_BlitSurface (sprite[posTorreta+16].img, NULL, superficie, &dest);
}

void EnemigoConTorreta::iniciar () {
	switch (tipoElemento) {
		case TANQUE:
			impactosParaMorir = IMPACTOS_TANQUE;
		break;
		case YATE:
			impactosParaMorir = IMPACTOS_YATE;
		break;
	}
	activo = 0;
	ciclos = 0;
	numMovimientos = 0;
	movimientoActual = -1;
	tipoRegalo = AG_NONE;
}


/*****************************
**
** CLASE TANQUE
**
******************************/

Tanque::Tanque()  : EnemigoConTorreta () {
	impactosParaMorir = IMPACTOS_YATE;
}

Tanque::Tanque(int nf)  : EnemigoConTorreta (nf) {
	impactosParaMorir = IMPACTOS_YATE;
}

/*****************************
**
** CLASE YATE
**
******************************/

Yate::Yate()  : EnemigoConTorreta () {
	impactosParaMorir = IMPACTOS_YATE;
}

Yate::Yate(int nf)  : EnemigoConTorreta (nf) {
	impactosParaMorir = IMPACTOS_YATE;
}

/*****************************
**
** CLASE CONSTRUCCIONES
**
******************************/

Construcciones::Construcciones()  : Enemigo () {
	impactosParaMorir = 1;
	impacto = 0;
}

Construcciones::Construcciones(int nf)  : Enemigo (nf) {
	impactosParaMorir = 1;
	impacto = 0;
}

void Construcciones::mover() {
	ciclos++;
	addY(2);
}

void Construcciones::iniciar() {
	activo = 0;
	ciclos = 0;
	numDisparos = 0;
	disparoActual = -1;
	if (getTipoElemento() == CASETA) {
		setImpactosParaMorir(IMPACTOS_CASETA);
	} else { // cualquier barrera
		setImpactosParaMorir(IMPACTOS_BARRERA);
	}
	impacto = 0;
	selFrame(0);
	tipoRegalo = AG_NONE;
}

int Construcciones::addImpactosParaMorir (Audio *sonido, Juego *juego, Explosion *exp) { 
	int ipm;
	impactosParaMorir--; 
	ipm = impactosParaMorir;
	setImpacto(1);
	if ( impactosParaMorir < 1  ) {
		if (getTipoElemento() == CASETA) {
			sonido->playSonido(SND_EXPCASETA);
			juego->addPuntos(PUNTOS_CASETA);
			juego->setHacerClip(0);		
			setActivo(0);
			iniciar();
			exp->crearExplosion(EXP_CASETA, getCentroX(), getCentroY());	
		} else { // cualquier barrera
			sonido->playSonido(SND_EXPLOSION);
			juego->addPuntos(PUNTOS_BARRERAS);
			juego->setHacerClip(0);		
			setActivo(0);
			iniciar();
			exp->crearExplosion(EXP_BARRERA, getCentroX(), getCentroY());
		}
		
	}
	return ipm;
}

void Construcciones::dibujar (SDL_Surface *superficie) {
	int fr;
	bool impactado = 0;
	SDL_Rect dest;
	dest.x = (int)posX;
	dest.y = (int)posY;

	int cicloEntreFrames;
	
	if (getTipoElemento() == CASETA) {
		cicloEntreFrames = 3;
	} else {
		cicloEntreFrames = 7;
	}
	
	if ( ciclos % cicloEntreFrames == 0 ) {
		if (actualFrame == 0) {
			fr = 1;
		} else {
			fr = 0;
		}
		selFrame (fr);
	}
	if ( (impacto == 1) && (getTipoElemento() == CASETA) ){
		impacto = 0;
		selFrame (2);
		impactado = 1;
	}
	SDL_SetAlpha(sprite[actualFrame].img, SDL_SRCALPHA, alpha);
	SDL_BlitSurface (sprite[actualFrame].img, NULL, superficie, &dest);

}


/*****************************
**
** CLASE MARINOS
**
******************************/

Mina::Mina()  : Enemigo () {
	impactosParaMorir = 1;
	impacto = 0;
	if (rand() % 2 == 0) {
		dirX = 2;
	} else {
		dirX = -2;
	}
}

Mina::Mina(int nf)  : Enemigo (nf) {
	impactosParaMorir = 1;
	impacto = 0;
	dirX = 2;
}

void Mina::mover() {
	ciclos++;
	addX(dirX);
}

void Mina::iniciar() {
	activo = 0;
	ciclos = 0;
	numDisparos = 0;
	disparoActual = -1;
	setImpactosParaMorir(IMPACTOS_MINA);
	impacto = 0;
	selFrame(0);
	tipoRegalo = AG_NONE;
}

int Mina::addImpactosParaMorir (Audio *sonido, Juego *juego, Explosion *exp) { 
	int ipm;
	impactosParaMorir--; 
	ipm = impactosParaMorir;
	setImpacto(1);
	if ( impactosParaMorir < 1  ) {
		sonido->playSonido(SND_EXPLOSION);
		//juego->addPuntos(PUNTOS_MARINOS);
		juego->setHacerClip(0);		
		setActivo(0);
		iniciar();
		//exp->crearExplosion(EXP_CANGREBOT, getX(), getY());				
	}
	return ipm;
}

void Mina::dibujar (SDL_Surface *superficie) {
	int fr;
	SDL_Rect dest;
	dest.x = (int)posX;
	dest.y = (int)posY;
	if (actualFrame != 9) {
		setFantasma(1);
	} else {
		setFantasma(0);
	}
	if ( ciclos % 4 == 0 ) {
		if (ciclos > 100) {
			fr = actualFrame - 1;	
		} else {
			fr = actualFrame + 1;
			if ( fr > 9 ) {
				fr = 9;
			}
		}

		if (fr < 0) {
			iniciar();
			fr = 0;
		}

		selFrame (fr);
	}

	SDL_SetAlpha(sprite[actualFrame].img, SDL_SRCALPHA, alpha);
	SDL_BlitSurface (sprite[actualFrame].img, NULL, superficie, &dest);

}

/*****************************
**
** CLASE AGREGADOR
**
******************************/

Agregador::Agregador()  : Elemento () {
	ciclos = 0;
	tipo = 0;
	dx = 4;
	dy = 4;
	
}

Agregador::Agregador(int nf)  : Elemento (nf) {
	ciclos = 0;
	tipo = 0;
	dx = 4;
	dy = 4;
	
}

void Agregador::posicionAleatoria() {
	int x;

	// posicion aleatoria
	x = rand()%360;
	//if ( x > 360 ) {
	//	x = (int) x % 360;
	//}
	if ( x < 150 ) {
		x = 150;
	}
	setX(x);
	setY(-getAlto()); 

}

void Agregador::mover () {
	ciclos++;
	addX(dx);
	addY(dy);
	
	if (ciclos < 400) {
		if (ciclos >13) {
		//if (getY()>5) {
			if ( getX() >= ( 515 - getAncho() ) || getX() < 5 )
				dx = -dx;
		
			if ( getY() >= (475 - getAlto()  ) || getY() < 5)
				dy = -dy;
		}
	} else {
		if ( fueraLimitesDePantalla() ) {
			iniciar();
			setActivo(0);
		}
	}
	
}

void Agregador::dibujar (SDL_Surface *superficie) {
	SDL_Rect dest;
	dest.x = (int)posX;
	dest.y = (int)posY;
	if ( ciclos % 3 == 0 ) {
		actualFrame++;
	}
	if (actualFrame > 8*(tipo+1)) {
		actualFrame = 9*tipo;
	}
	SDL_SetAlpha(sprite[actualFrame].img, SDL_SRCALPHA, alpha);
	SDL_BlitSurface (sprite[actualFrame].img, NULL, superficie, &dest);
}

void Agregador::iniciar () {
	dx = 4;
	dy = 4;
	activo = 0;
	ciclos = 0;
	posicionAleatoria();
}



/*****************************
**
** CLASE JEFEFASE1
**
******************************/

JefeFase1::JefeFase1(SDL_Surface *screen)  {
	char tmpch[127];
	Frame frCuerpo, frCuerpoImpacto, frPataIz0, frPataIz1, frPataIz2, frPataDe0, frPataDe1, frPataDe2;
	Frame frOjos0, frOjos1, frOjos2, frOjos3, frSombra;
	Frame frBrazoIz0, frBrazoIz1, frBrazoIz2, frBrazoIz3, frBrazoIz4;
	Frame frBrazoDe0, frBrazoDe1, frBrazoDe2, frBrazoDe3, frBrazoDe4;
	impactosParaMorir = IMPACTOS_JEFE1;
	impacto = 0;
	ciclos = 0;
	cerrarOjos = 0;
	conMovimiento  = 1;
	conMovimiento_brazo = 1;
	activo = 1;
	llamada = 0;
	sw_primerallamada = 1;
	tiempoBloqueoDisparosEnemigos = 0;
	
	direccionX = 3;
	direccionX_aux = 3;
	
	// creamos la barra de vida
	
	barra = new BarraVidaEnemigo(screen, "Cangrejo", IMPACTOS_JEFE1, 460, 20);
	
	// cargamos imágenes
	
	sprintf (tmpch, "%s/jefe1.tim", DATA_DIR);
	
	sombra = new Elemento();	
	frSombra.loadFromRAR(tmpch, "cang_sombra.timp");
	sombra->addFrame(frSombra);
	
	cuerpo = new Elemento(2);	
	frCuerpo.loadFromRAR(tmpch, "cang_boca.timp");
	cuerpo->addFrame(frCuerpo);
	
	frCuerpoImpacto.loadFromRAR(tmpch, "cang_boca_impacto.timp");
	cuerpo->addFrame(frCuerpoImpacto);
	
	pata_iz = new Elemento(3);
	frPataIz0.loadFromRAR(tmpch, "cang_pata_iz0.timp");
	pata_iz->addFrame(frPataIz0);
	frPataIz1.loadFromRAR(tmpch, "cang_pata_iz1.timp");
	pata_iz->addFrame(frPataIz1);
	frPataIz2.loadFromRAR(tmpch, "cang_pata_iz2.timp");
	pata_iz->addFrame(frPataIz2);
	
	pata_de = new Elemento(3);
	frPataDe0.loadFromRAR(tmpch, "cang_pata_de0.timp");
	pata_de->addFrame(frPataDe0);
	frPataDe1.loadFromRAR(tmpch, "cang_pata_de1.timp");
	pata_de->addFrame(frPataDe1);
	frPataDe2.loadFromRAR(tmpch, "cang_pata_de2.timp");
	pata_de->addFrame(frPataDe2);
	
	ojos = new Elemento(4);
	frOjos0.loadFromRAR(tmpch, "cang_ojos_ab.timp");
	ojos->addFrame(frOjos0);
	frOjos1.loadFromRAR(tmpch, "cang_ojos_ce.timp");
	ojos->addFrame(frOjos1);
	frOjos2.loadFromRAR(tmpch, "cang_ojos_iz.timp");
	ojos->addFrame(frOjos2);
	frOjos3.loadFromRAR(tmpch, "cang_ojos_de.timp");
	ojos->addFrame(frOjos3);
	
	
	brazo_de = new Elemento(5);
	frBrazoDe0.loadFromRAR(tmpch, "cang_brazo_de0.timp");
	brazo_de->addFrame(frBrazoDe0);
	frBrazoDe1.loadFromRAR(tmpch, "cang_brazo_de1.timp");
	brazo_de->addFrame(frBrazoDe1);
	frBrazoDe2.loadFromRAR(tmpch, "cang_brazo_de2.timp");
	brazo_de->addFrame(frBrazoDe2);
	frBrazoDe3.loadFromRAR(tmpch, "cang_brazo_de3.timp");
	brazo_de->addFrame(frBrazoDe3);
	frBrazoDe4.loadFromRAR(tmpch, "cang_brazo_de4.timp");
	brazo_de->addFrame(frBrazoDe4);
	

	brazo_iz = new Elemento(5);
	frBrazoIz0.loadFromRAR(tmpch, "cang_brazo_iz0.timp");
	brazo_iz->addFrame(frBrazoIz0);
	frBrazoIz1.loadFromRAR(tmpch, "cang_brazo_iz1.timp");
	brazo_iz->addFrame(frBrazoIz1);
	frBrazoIz2.loadFromRAR(tmpch, "cang_brazo_iz2.timp");
	brazo_iz->addFrame(frBrazoIz2);
	frBrazoIz3.loadFromRAR(tmpch, "cang_brazo_iz3.timp");
	brazo_iz->addFrame(frBrazoIz3);
	frBrazoIz4.loadFromRAR(tmpch, "cang_brazo_iz4.timp");
	brazo_iz->addFrame(frBrazoIz4);

	posicionInicial();
}

void JefeFase1::posicionInicial() {
	
	posicionX = 150;
	posicionY = -25;
	
	sombra->setX(posicionX+60);
	sombra->setY(posicionY+169);
	
	cuerpo->setX(posicionX+78);
	cuerpo->setY(posicionY+130);
	
	pata_iz->setX(posicionX+20);
	pata_iz->setY(posicionY+140);
	
	pata_de->setX(posicionX+218);
	pata_de->setY(posicionY+140);
	
	ojos->setX(posicionX+119);
	ojos->setY(posicionY+89);
	
	brazo_de->setX(posicionX+165);
	brazo_de->setY(posicionY+25);
	
	brazo_iz->setX(posicionX+45);
	brazo_iz->setY(posicionY+20);	
}

void JefeFase1::addX(int x) {
	posicionX += x;
	pata_de->addX(x);
	pata_iz->addX(x);
	cuerpo->addX(x);
	brazo_de->addX(x);
	brazo_iz->addX(x);
	sombra->addX(x);
	ojos->addX(x);	
}

void JefeFase1::mover() {
	
	if (ciclos%100==0) {
		if (conMovimiento == 1) {
			conMovimiento = 0;
		} else {
			conMovimiento = 1;
			direccionX = direccionX_aux;
		}
	}
	
	if ((conMovimiento == 1) && (llamada==0) ) {
		if (posicionX < 0) {
			direccionX = 3;
			direccionX_aux = 3;
		}
		if (posicionX > 200) {
			direccionX = -3;
			direccionX_aux = -3;
		}
		addX(direccionX);
	} else {
		direccionX = 0;
	}
}

void JefeFase1::dibujar(SDL_Surface *superficie) {
	if (finLlamada == 1) { // establecer fin de llamada a solo 1 ciclo
		finLlamada = 0;
	}
	
	ciclos++;
	
	barra->dibujar(superficie);

	if (ciclos%140==0) { 
		if (conMovimiento_brazo == 1) {
			conMovimiento_brazo = 0;
		} else {
			conMovimiento_brazo = 1;
		}
	}
	
	
	if (ciclos%3==0) { 
		if (direccionX!=0) {
			pata_iz->selSiguienteFrame();
			pata_de->selSiguienteFrame();
		}
	}
	
	if (llamada ==0) {
		if (conMovimiento_brazo == 1) {
			if (ciclos%7==0) { 
				brazo_de->selSiguienteFrame_ReboteHasta(3);
			}
			if (ciclos%6==0) { 
				brazo_iz->selSiguienteFrame_ReboteHasta(3);
			}
		}
	} else {
		if (ciclos%3==0) { 
			if (brazo_de->getActualFrame() ==3) {
				brazo_de->selFrame(4);
			} else {
				brazo_de->selFrame(3);
			}
			if (brazo_iz->getActualFrame() ==3) {
				brazo_iz->selFrame(4);
			} else {
				brazo_iz->selFrame(3);
			}
		}
		if (ciclos%100==0) { 
			llamada = 0;
			finLlamada = 1;
			brazo_de->selFrame(0);
			brazo_iz->selFrame(0);
		}
	}
	if (ciclos%70==0) {
		cerrarOjos = 1;
	}

	sombra->dibujar(superficie);
	
	pata_iz->dibujar(superficie);
	pata_de->dibujar(superficie);
	brazo_de->dibujar(superficie);
	brazo_iz->dibujar(superficie);
	if (impacto==1) {
		cuerpo->selFrame(1);
		impacto = 0;
	} else {
		cuerpo->selFrame(0);
	}
	cuerpo->dibujar(superficie);
	if (direccionX>0) {
		ojos->selFrame(2);
	} else if (direccionX<0) {
		ojos->selFrame(3);
	} else {
		if (cerrarOjos > 0) {
			ojos->selFrame(1);
			cerrarOjos++;
			if (cerrarOjos>5) {
				cerrarOjos = 0;
			}
		} else {
			ojos->selFrame(0);
		}
	}
	
	ojos->dibujar(superficie);
}

int JefeFase1::colision(Elemento *elem) {
	return cuerpo->colision(elem);
}

int JefeFase1::colisionConJugador(Jugador *j) {
	return cuerpo->colision(j);
}

int JefeFase1::addImpactosParaMorir (Audio *sonido, Juego *juego, Explosion *exp) { 
	int ipm;
	impactosParaMorir--; 
	barra->actualizarVida(impactosParaMorir);
	ipm = impactosParaMorir;
	setImpacto(1);
	if ( impactosParaMorir <= 0  ) {
		sonido->playSonido(SND_EXP_JEFES);
		juego->addPuntos(PUNTOS_JEFE1);
		juego->setHacerClip(0);		
		setActivo(0);
		exp->crearExplosionJefe1(posicionX, posicionY);
	}
	return ipm;
}

void JefeFase1::crearDisparo(DisparosEnemigos *disp_ene, Audio *son) {
	if (tiempoBloqueoDisparosEnemigos>0) {
		tiempoBloqueoDisparosEnemigos--;
	} else {
		switch (ciclos) {
			case 30:
				disp_ene->crearDisparo (NULL, DISP_RAFAGA_GR, cuerpo->getCentroX(), cuerpo->getCentroY(), 0, 0);
			break;
			case 80:
				disp_ene->crearDisparo (DISPARO_POMPAPEQ, DISP_DIRIGIDO, cuerpo->getCentroX(), cuerpo->getCentroY(), 50, 0);
			break;
			case 90:
				disp_ene->crearDisparo (DISPARO_POMPAPEQ, DISP_DIRIGIDO, cuerpo->getCentroX(), cuerpo->getCentroY(), 450, 0);
			break;
			case 110:
				disp_ene->crearDisparo (DISPARO_POMPAGR, DISP_DIRIGIDO, cuerpo->getCentroX(), cuerpo->getCentroY(), 450, 0);
			break;
			case 140:
				disp_ene->crearDisparo (DISPARO_GR_AZUL, DISP_5ABAJO, cuerpo->getCentroX(), cuerpo->getCentroY(), 0, 0);
			break;
			case 160:
				disp_ene->crearDisparo (DISPARO_ROT, DISP_2ABAJO, cuerpo->getCentroX(), cuerpo->getCentroY(), 0, 0);
			break;
			case 180:
				disp_ene->crearDisparo (DISPARO_ROT, DISP_5ABAJO, cuerpo->getCentroX(), cuerpo->getCentroY(), 0, 0);
			break;
			case 200:
				disp_ene->crearDisparo (DISPARO_ROT, DISP_2ABAJO, cuerpo->getCentroX(), cuerpo->getCentroY(), 0, 0);
			break;
			case 210:
				disp_ene->crearDisparo (DISPARO_POMPAGR, DISP_DIRIGIDO, cuerpo->getCentroX(), cuerpo->getCentroY(), 100, 0);
			break;
			case 240:
				disp_ene->crearDisparo (DISPARO_POMPAGR, DISP_DIRIGIDO, cuerpo->getCentroX(), cuerpo->getCentroY(), 550, 0);
			break;
			case 260:
				disp_ene->crearDisparo (NULL, DISP_RAFAGA_MED, cuerpo->getCentroX(), cuerpo->getCentroY(), 0, 0);
			break;
			case 300:
				disp_ene->crearDisparo (DISPARO_PEQ, DISP_3ABAJO, cuerpo->getCentroX(), cuerpo->getCentroY(), 0, 0);
			break;
			case 330:
				disp_ene->crearDisparo (DISPARO_PEQ, DISP_3ABAJO, cuerpo->getCentroX(), cuerpo->getCentroY(), 0, 0);
			break;
			case 360:
				disp_ene->crearDisparo (DISPARO_ROT, DISP_6LATERALES, cuerpo->getCentroX(), cuerpo->getCentroY(), 0, 0);
			break;
			case 380:
				disp_ene->crearDisparo (DISPARO_ROT, DISP_5ABAJO, cuerpo->getCentroX(), cuerpo->getCentroY(), 0, 0);
			break;
			case 390:
				disp_ene->crearDisparo (DISPARO_GR_AZUL, DISP_3ABAJO_CERRADO, cuerpo->getCentroX(), cuerpo->getCentroY(), 0, 0);
			break;
			/*
			case 430:
				disp_ene->crearDisparo (DISPARO_POMPAGR, DISP_DIRIGIDO, cuerpo->getCentroX(), cuerpo->getCentroY(), 550, 0);
			break;
			case 440:
				disp_ene->crearDisparo (DISPARO_ROT, DISP_6LATERALES, cuerpo->getCentroX(), cuerpo->getCentroY(), 0, 0);
			break;
			case 460:
				disp_ene->crearDisparo (DISPARO_ROT, DISP_5ABAJO, cuerpo->getCentroX(), cuerpo->getCentroY(), 0, 0);
			break;*/
			case 400:
				llamada = 1;
				ciclos = 0;
				son->playSonido(SND_CASTANUELAS);
			break;
		}
	}
}

void JefeFase1::unLoad() {
	cuerpo->unLoad();
	brazo_iz->unLoad();
	brazo_de->unLoad();
	pata_iz->unLoad();
	pata_de->unLoad();
	ojos->unLoad();
	sombra->unLoad();
}

void JefeFase1::iniciar() {
	impactosParaMorir = IMPACTOS_JEFE1;
	tiempoBloqueoDisparosEnemigos = 0;
	impacto = 0;
	ciclos = 0;
	cerrarOjos = 0;
	conMovimiento  = 1;
	conMovimiento_brazo = 1;
	activo = 1;
	llamada = 0;
	sw_primerallamada = 1;
	
	barra->setTotalVida(IMPACTOS_JEFE1);
	
	posicionInicial();
	
	direccionX = 3;
	direccionX_aux = 3;
}
	
	
/*****************************
**
** CLASE TORREBALNEARIO
**
******************************/

TorreBalneario::TorreBalneario()  : Enemigo () {
	iniciar();
}

TorreBalneario::TorreBalneario(int nf)  : Enemigo (nf) {
	iniciar();
}


int TorreBalneario::addImpactosParaMorir (Audio *sonido, Juego *juego, Explosion *exp) { 
	int ipm;
	impactosParaMorir--; 
	ipm = impactosParaMorir;
	impacto = 1;
	if ( impactosParaMorir < 1  ) {
		sonido->playSonido(SND_EXPLOSION);
		juego->addPuntos(PUNTOS_TORREBALNEARIO);
		juego->setHacerClip(0);		
		setActivo(0);
		exp->crearExplosion(EXP_TORRE, getCentroX(), getCentroY());
	}
	return ipm;
}

void TorreBalneario::iniciar () {
	activo = 0;
	ciclos = 0;
	numDisparos = 0;
	disparoActual = -1;
	setImpactosParaMorir(IMPACTOS_TORREBALNEARIO);
	abrir = 0;
	cerrar = 0;
	torreAbierta = 0;
	tiempoTorreAbierta = 180;
	tipoRegalo = AG_NONE;
	actualFrame = 0;
	impacto=0;
	movimiento = 1;
}


void TorreBalneario::dibujar (SDL_Surface *superficie) {
	SDL_Rect dest;
	dest.x = (int)posX;
	dest.y = (int)posY;
	int auxFrame = -1;
	ciclos++;
	if (impacto == 1) {
		auxFrame = actualFrame;
		selFrame(7);
		impacto = 0;
	} else {
		//if ( (torreAbierta == 1) && (abrir==0) && (cerrar==0) ) {
		if (torreAbierta == 1) {
			if (actualFrame<3) {
				setFantasma(1);
			} else {				
				setFantasma(0);
			}
		} else {
			setFantasma(1);
		}
		
		if (torreAbierta == 1) {
			//printf("mov: %d, abrir: %d, cerrar: %d\n", movimiento, abrir, cerrar);
			//setFantasma(0);
			tiempoTorreAbierta--;
			if (tiempoTorreAbierta < 0) {
				tiempoTorreAbierta = 180;
				cerrarTorre();
			}
		}
		
		if (abrir==1) {
			if (ciclos%3==0) {
				actualFrame += movimiento;
				if (actualFrame>=6) {
					movimiento = -1;
				} else if (actualFrame<=3) {
					movimiento = 1;
				}
			}
		}
		if (cerrar==1) {
			if (ciclos%3==0) {
				actualFrame--;
				if (actualFrame==0) {
					actualFrame = 0;
					cerrar = 0;
				}
			}				
		}
		
		/*if (actualFrame==4) {
			actualFrame = 3;
		}*/
	}
	SDL_BlitSurface (sprite[actualFrame].img, NULL, superficie, &dest);
	if (auxFrame!=-1) {
		selFrame(auxFrame);
	}
}

/*****************************
**
** CLASE JEFEFASE3
**
******************************/

JefeFase3::JefeFase3()  : Enemigo () {
	impactosParaMorir = IMPACTOS_JEFE3;
	tiempoFiesta = TIEMPOFIESTAFINAL3;
	tipoMov = 1;
	impacto = 0;
	activo = 1;
	actualMovimientoPrincipal = 0;
	ciclos = 0;
	enAtaque = false;
	ataqueActual = 1;
	ciclosRealizarAtaque = 140;
	fiesta = false;
	tiempoBloqueoDisparosEnemigos = 0;
	
	fuego0 = new Elemento(3);
	humo = new Elemento(3);
}

JefeFase3::JefeFase3(int nf)  : Enemigo (nf) {
	impactosParaMorir = IMPACTOS_JEFE3;
	tiempoFiesta = TIEMPOFIESTAFINAL3;
	tipoMov = 1;
	impacto = 0;
	activo = 1;
	actualMovimientoPrincipal = 0;
	ciclos = 0;
	enAtaque = false;
	ataqueActual = 1;
	ciclosRealizarAtaque = 140;
	fiesta = false;
	tiempoBloqueoDisparosEnemigos = 0;
	
	fuego0 = new Elemento(3);
	humo = new Elemento(3);
}

void JefeFase3::crearBarraVida (SDL_Surface *screen, char texto[32], int tv, int posX, int posY) { 
	// creamos la barra de vida
	barra = new BarraVidaEnemigo(screen, texto, tv, posX, posY);
}

int JefeFase3::addImpactosParaMorir (Audio *sonido, Juego *juego, Explosion *exp) { 
	int ipm;
	impactosParaMorir--; 
	barra->actualizarVida(impactosParaMorir);
	ipm = impactosParaMorir;
	setImpacto(1);
	if ( impactosParaMorir <= 0  ) {
		sonido->playSonido(SND_EXP_JEFES);
		juego->addPuntos(PUNTOS_JEFE2);
		juego->setHacerClip(0);	
		setFantasma(1);
		exp->crearExplosion(EXP_NAVEMED, getCentroX()-80, getCentroY()-40);
		exp->crearExplosion(EXP_MEDIANA, getCentroX()+30, getCentroY()-80);
		exp->crearExplosion(EXP_NAVEMED, getCentroX()-30, getCentroY()-80);
		exp->crearExplosion(EXP_NAVEGRANDE, getCentroX()+30, getCentroY()-20);
		exp->crearExplosion(EXP_NAVEGRANDE, getCentroX()+20, getCentroY()-40);
		exp->crearExplosion(EXP_MEDIANA1, getCentroX()-20, getCentroY()-30);
	}
	return ipm;
}

void JefeFase3::crearDisparo(DisparosEnemigos *disp_ene) {
	if (tiempoBloqueoDisparosEnemigos>0) {
		tiempoBloqueoDisparosEnemigos--;
	} else {
		if ( (!fiesta) && ( getFantasma() == 0 ) ) {
			switch (dios) {
				case DIOS_BACO:
					switch (ciclos) {
							case 30:
								disp_ene->crearDisparo (NULL, DISP_RAFAGA_GR, getCentroX(), getCentroY(), 0, 0);
							break;
							case 80:
								disp_ene->crearDisparo (DISPARO_GR_AZUL, DISP_5ABAJO, getCentroX(), getCentroY(), 0, 0);
							break;
							case 180:
								disp_ene->crearDisparo (DISPARO_ROT, DISP_5ABAJO, getCentroX(), getCentroY(), 0, 0);
							break;
							case 200:
								disp_ene->crearDisparo (DISPARO_ROT, DISP_5ABAJO, getCentroX(), getCentroY(), 0, 0);
							break;
							case 330:
								disp_ene->crearDisparo (DISPARO_PEQ, DISP_3ABAJO, getCentroX(), getCentroY(), 0, 0);
							break;
							case 410:
								disp_ene->crearDisparo (DISPARO_GR_AZUL, DISP_3ABAJO_CERRADO, getCentroX(), getCentroY(), 0, 0);
							break;
							case 450:
								disp_ene->crearDisparo (DISPARO_ROT, DISP_5ABAJO, getCentroX(), getCentroY(), 0, 0);
							break;
							case 470:
								disp_ene->crearDisparo (DISPARO_ROT, DISP_3ABAJO_CERRADO, getCentroX(), getCentroY(), 0, 0);
							break;
						}			
				break;
				case DIOS_HERCULES:
					switch (ciclos) {
							case 20:
								disp_ene->crearDisparo (DISPARO_ROT, DISP_2ABAJO, getCentroX(), getCentroY(), 0, 0);
							break;
							case 40:
								disp_ene->crearDisparo (DISPARO_ROT, DISP_2ABAJO, getCentroX(), getCentroY(), 0, 0);
							break;
							case 60:
								disp_ene->crearDisparo (DISPARO_ROT, DISP_2ABAJO, getCentroX(), getCentroY(), 0, 0);
							break;
							case 160:
								disp_ene->crearDisparo (DISPARO_PEQ, DISP_2ABAJO, getCentroX(), getCentroY(), 0, 0);
							break;
							case 180:
								disp_ene->crearDisparo (DISPARO_PEQ, DISP_2ABAJO, getCentroX(), getCentroY(), 0, 0);
							break;
							case 200:
								disp_ene->crearDisparo (DISPARO_ROT, DISP_2ABAJO, getCentroX(), getCentroY(), 0, 0);
							break;
							case 260:
								disp_ene->crearDisparo (NULL, DISP_RAFAGA_MED, getCentroX(), getCentroY(), 0, 0);
							break;
							case 360:
								disp_ene->crearDisparo (DISPARO_ROT, DISP_8CIRCULAR, getCentroX(),getCentroY(), 0, 0);
							break;
							case 400:
								disp_ene->crearDisparo (DISPARO_ROT, DISP_8CIRCULAR, getCentroX(),getCentroY(), 0, 0);
							break;
							case 440:
								disp_ene->crearDisparo (DISPARO_PEQ, DISP_5ABAJO, getCentroX(), getCentroY(), 0, 0);
							break;
							case 460:
								disp_ene->crearDisparo (DISPARO_PEQ, DISP_5ABAJO, getCentroX(), getCentroY(), 0, 0);
							break;
						}			
				break;
				case DIOS_NEPTUNO:
					switch (ciclos) {
							case 100:
								disp_ene->crearDisparo (DISPARO_GR, DISP_3ABAJO, getCentroX(), getCentroY(), 0, 0);
							break;
							case 120:
								disp_ene->crearDisparo (DISPARO_GR, DISP_3ABAJO, getCentroX(), getCentroY(), 0, 0);
							break;
							case 160:
								disp_ene->crearDisparo (DISPARO_ROT, DISP_5ABAJO, getCentroX(), getCentroY(), 0, 0);
							break;
							case 180:
								disp_ene->crearDisparo (DISPARO_ROT, DISP_5ABAJO, getCentroX(), getCentroY(), 0, 0);
							break;
							case 260:
								disp_ene->crearDisparo (DISPARO_GR_AZUL, DISP_6LATERALES, getCentroX(),getCentroY(), 0, 0);
							break;
							case 280:
								disp_ene->crearDisparo (DISPARO_GR_AZUL, DISP_4OBLICUO, getCentroX(),getCentroY(), 0, 0);
							break;
							case 300:
								disp_ene->crearDisparo (DISPARO_GR_AZUL, DISP_4CRUZ, getCentroX(),getCentroY(), 0, 0);
							break;
							case 320:
								disp_ene->crearDisparo (DISPARO_GR_AZUL, DISP_4OBLICUO, getCentroX(),getCentroY(), 0, 0);
							break;
							case 340:
								disp_ene->crearDisparo (DISPARO_GR_AZUL, DISP_4CRUZ, getCentroX(),getCentroY(), 0, 0);
							break;
							case 380:
								disp_ene->crearDisparo (DISPARO_GR_AZUL, DISP_6LATERALES, getCentroX(), getCentroY(), 0, 0);
							break;
							case 410:
								disp_ene->crearDisparo (NULL, DISP_RAFAGA_MED, getCentroX(), getCentroY(), 0, 0);
							break;
							case 440:
								disp_ene->crearDisparo (DISPARO_GR, DISP_5ABAJO, getCentroX(), getCentroY(), 0, 0);
							break;
							case 460:
								disp_ene->crearDisparo (DISPARO_GR, DISP_5ABAJO, getCentroX(), getCentroY(), 0, 0);
							break;
						}			
				break;
			}
		}
	}
}

void JefeFase3::dibujar (SDL_Surface *superficie) {
	SDL_Rect dest;
	dest.x = (int)posX;
	dest.y = (int)posY;
	
	if (ciclos > 480) {
		ciclos = 0;
	}
	
	ciclos++;
	
	barra->dibujar(superficie);
	
	if ( getFantasma() == 1 ) {
	
		actualFrame = getFrameExplotado();
		
	} else if (fiesta) {
		
		tiempoFiesta--;
		if (tiempoFiesta < 0) {
			if (ciclos%3==0){
				actualFrame--;
				if (actualFrame<fiestaDesde) {
					tipoMov = 1;
					tiempoFiesta = TIEMPOFIESTAFINAL3;
					setFiesta(false);
					enAtaque = false;
					actualFrame = tilesMovimientoPrincipal[actualMovimientoPrincipal];
				}
			}
		} else {
			if (ciclos%3==0){
				actualFrame += tipoMov;
				if (actualFrame>=fiestaHasta-1) {
					tipoMov = -1;
				} else if (actualFrame<=fiestaDesde+numFramesMovimientoFiesta) {
					tipoMov = 1;
				}
			}
		}

	} else {
		
		if (enAtaque) {
			if (ciclos%3==0){
				actualFrame++;
				switch(ataqueActual) {
					case 1:
						if (ataque1Hasta == actualFrame) {
							enAtaque = false;
						}
					break;
					case 2:
						if (ataque2Hasta == actualFrame) {
							enAtaque = false;
						}
					break;
					case 3:
						if (ataque3Hasta == actualFrame) {
							enAtaque = false;
						}
					break;
				}
			}
		} else {
			if (ciclos%7==0){
				if (actualMovimientoPrincipal<3) {
					actualMovimientoPrincipal++;
				} else {
					actualMovimientoPrincipal = 0;
				}
				actualFrame = tilesMovimientoPrincipal[actualMovimientoPrincipal];
				//printf("%d\n", actualFrame);
			}
		}
		
		if (ciclos%ciclosRealizarAtaque==0) {
			enAtaque = true;
			ataqueActual = rand()%3+1;
			switch(ataqueActual) {
				case 1:
					actualFrame = ataque1Desde;
				break;
				case 2:
					actualFrame = ataque2Desde;
				break;
				case 3:
					actualFrame = ataque3Desde;
				break;	
			}
		}
		
	}
	
	SDL_BlitSurface (sprite[actualFrame].img, NULL, superficie, &dest);
	
	if (getFantasma()==1) {
		if (ciclos%3==0) {
			fuego0->selSiguienteFrame();
			humo->selSiguienteFrame();
		}
		switch (dios) {
			case DIOS_BACO:
				fuego0->setX(posX+85);
				fuego0->setY(posY+65);
			break;
			case DIOS_HERCULES:
				fuego0->setX(posX+70);
				fuego0->setY(posY+60);
			break;
			case DIOS_NEPTUNO:
				fuego0->setX(posX+110);
				fuego0->setY(posY+80);
			break;
		}
		fuego0->dibujar(superficie);
		
		switch (dios) {
			case DIOS_BACO:
				humo->setX(posX+125);
				humo->setY(posY+40);
			break;
			case DIOS_HERCULES:
				humo->setX(posX+110);
				humo->setY(posY+45);
			break;
			case DIOS_NEPTUNO:
				humo->setX(posX+205);
				humo->setY(posY+70);
			break;
		}
		humo->dibujar(superficie);
	}
	
}

void JefeFase3::iniciar () {
	setFantasma(0);
	fiesta = false;
	tiempoFiesta = TIEMPOFIESTAFINAL3;
	tipoMov = 1;
	impactosParaMorir = IMPACTOS_JEFE3;
	barra->setTotalVida(IMPACTOS_JEFE3);
	impacto = 0;
	activo = 1;
	actualMovimientoPrincipal = 0;
	ciclos = 0;
	enAtaque = false;
	ataqueActual = 1;
	ciclosRealizarAtaque = 140;
	tiempoBloqueoDisparosEnemigos = 0;
}


/*****************************
**
** CLASE JEFEFASE2
**
******************************/

JefeFase2::JefeFase2(SDL_Surface *screen)  {
	char tmpch[127];
	
	impactosParaMorir = IMPACTOS_JEFE2;
	impacto = 0;
	ciclos = 0;
	activo = 1;
	tiempoBloqueoDisparosEnemigos = 0;
	
	// creamos la barra de vida
	
	barra = new BarraVidaEnemigo(screen, "Pulpo", IMPACTOS_JEFE2, 460, 20);
	
	// cargamos imágenes
	
	sprintf (tmpch, "%s/jefe2.tim", DATA_DIR);
	
	Frame frAlquitran0, frAlquitran1;
	frAlquitran0.loadFromRAR(tmpch, "alquitran0.timp");
	frAlquitran1.loadFromRAR(tmpch, "alquitran1.timp");
	
	int i;
	
	for (i=0; i<MAX_ALQUITRAN; i++) {
		alquitran1[i] = new Elemento();
		alquitran2[i] = new Elemento();
		switch(i%2) {
			case 0:
				alquitran1[i]->addFrame(frAlquitran0);
				alquitran2[i]->addFrame(frAlquitran0);
			break;
			case 1:
				alquitran1[i]->addFrame(frAlquitran1);
				alquitran2[i]->addFrame(frAlquitran1);
			break;
		}
	}
	
	Frame frMancha0, frMancha1, frMancha2, frMancha3;
	Frame frMancha4, frMancha5, frMancha6;

	mancha = new Elemento(7);
	frMancha0.loadFromRAR(tmpch, "mancha_0.timp");
	mancha->addFrame(frMancha0);
	frMancha1.loadFromRAR(tmpch, "mancha_1.timp");
	mancha->addFrame(frMancha1);
	frMancha2.loadFromRAR(tmpch, "mancha_2.timp");
	mancha->addFrame(frMancha2);
	frMancha3.loadFromRAR(tmpch, "mancha_3.timp");
	mancha->addFrame(frMancha3);
	frMancha4.loadFromRAR(tmpch, "mancha_4.timp");
	mancha->addFrame(frMancha4);
	frMancha5.loadFromRAR(tmpch, "mancha_5.timp");
	mancha->addFrame(frMancha5);
	frMancha6.loadFromRAR(tmpch, "mancha_6.timp");
	mancha->addFrame(frMancha6);
	
	Frame frPatas0, frPatas1, frPatas2, frPatas3;

	patas = new Elemento(5);
	frPatas0.loadFromRAR(tmpch, "patas_0.timp");
	patas->addFrame(frPatas0);
	frPatas1.loadFromRAR(tmpch, "patas_1.timp");
	patas->addFrame(frPatas1);
	frPatas2.loadFromRAR(tmpch, "patas_2.timp");
	patas->addFrame(frPatas2);
	frPatas3.loadFromRAR(tmpch, "patas_3.timp");
	patas->addFrame(frPatas3);
	
	Frame frCabeza0, frCabeza1, frCabeza2, frCabeza3, frCabeza4;
	
	cabeza = new Elemento(5);
	frCabeza0.loadFromRAR(tmpch, "cabeza_0.timp");
	cabeza->addFrame(frCabeza0);
	frCabeza1.loadFromRAR(tmpch, "cabeza_1.timp");
	cabeza->addFrame(frCabeza1);
	frCabeza2.loadFromRAR(tmpch, "cabeza_2.timp");
	cabeza->addFrame(frCabeza2);
	frCabeza3.loadFromRAR(tmpch, "cabeza_3.timp");
	cabeza->addFrame(frCabeza3);
	frCabeza4.loadFromRAR(tmpch, "cabeza_4.timp");
	cabeza->addFrame(frCabeza4);
	
	Frame frTent1_0, frTent1_1, frTent1_2, frTent1_3;
	Frame frTent1_4, frTent1_5, frTent1_6, frTent1_7;
	Frame frTent1_8, frTent1_9, frTent1_10, frTent1_11, frTent1_12;
	
	tent1 = new Elemento(13);
	frTent1_0.loadFromRAR(tmpch, "tent1_00.timp");
	tent1->addFrame(frTent1_0);
	frTent1_1.loadFromRAR(tmpch, "tent1_01.timp");
	tent1->addFrame(frTent1_1);
	frTent1_2.loadFromRAR(tmpch, "tent1_02.timp");
	tent1->addFrame(frTent1_2);
	frTent1_3.loadFromRAR(tmpch, "tent1_03.timp");
	tent1->addFrame(frTent1_3);
	frTent1_4.loadFromRAR(tmpch, "tent1_04.timp");
	tent1->addFrame(frTent1_4);
	frTent1_5.loadFromRAR(tmpch, "tent1_05.timp");
	tent1->addFrame(frTent1_5);
	frTent1_6.loadFromRAR(tmpch, "tent1_06.timp");
	tent1->addFrame(frTent1_6);
	frTent1_7.loadFromRAR(tmpch, "tent1_07.timp");
	tent1->addFrame(frTent1_7);
	frTent1_8.loadFromRAR(tmpch, "tent1_08.timp");
	tent1->addFrame(frTent1_8);
	frTent1_9.loadFromRAR(tmpch, "tent1_09.timp");
	tent1->addFrame(frTent1_9);
	frTent1_10.loadFromRAR(tmpch, "tent1_10.timp");
	tent1->addFrame(frTent1_10);
	frTent1_11.loadFromRAR(tmpch, "tent1_11.timp");
	tent1->addFrame(frTent1_11);
	frTent1_12.loadFromRAR(tmpch, "tent1_12.timp");
	tent1->addFrame(frTent1_12);
	
	tent1->setX(-80);
	tent1->setY(70);
	

	tent2 = new Elemento(13);
	frTent1_0.loadFromRAR(tmpch, "tent2_00.timp");
	tent2->addFrame(frTent1_0);
	frTent1_1.loadFromRAR(tmpch, "tent2_01.timp");
	tent2->addFrame(frTent1_1);
	frTent1_2.loadFromRAR(tmpch, "tent2_02.timp");
	tent2->addFrame(frTent1_2);
	frTent1_3.loadFromRAR(tmpch, "tent2_03.timp");
	tent2->addFrame(frTent1_3);
	frTent1_4.loadFromRAR(tmpch, "tent2_04.timp");
	tent2->addFrame(frTent1_4);
	frTent1_5.loadFromRAR(tmpch, "tent2_05.timp");
	tent2->addFrame(frTent1_5);
	frTent1_6.loadFromRAR(tmpch, "tent2_06.timp");
	tent2->addFrame(frTent1_6);
	frTent1_7.loadFromRAR(tmpch, "tent2_07.timp");
	tent2->addFrame(frTent1_7);
	frTent1_8.loadFromRAR(tmpch, "tent2_08.timp");
	tent2->addFrame(frTent1_8);
	frTent1_9.loadFromRAR(tmpch, "tent2_09.timp");
	tent2->addFrame(frTent1_9);
	frTent1_10.loadFromRAR(tmpch, "tent2_10.timp");
	tent2->addFrame(frTent1_10);
	frTent1_11.loadFromRAR(tmpch, "tent2_11.timp");
	tent2->addFrame(frTent1_11);
	frTent1_12.loadFromRAR(tmpch, "tent2_12.timp");
	tent2->addFrame(frTent1_12);
	
	tent2->setX(300);
	tent2->setY(110);
	

	tent3 = new Elemento(12);
	frTent1_0.loadFromRAR(tmpch, "tent3_00.timp");
	tent3->addFrame(frTent1_0);
	frTent1_1.loadFromRAR(tmpch, "tent3_01.timp");
	tent3->addFrame(frTent1_1);
	frTent1_2.loadFromRAR(tmpch, "tent3_02.timp");
	tent3->addFrame(frTent1_2);
	frTent1_3.loadFromRAR(tmpch, "tent3_03.timp");
	tent3->addFrame(frTent1_3);
	frTent1_4.loadFromRAR(tmpch, "tent3_04.timp");
	tent3->addFrame(frTent1_4);
	frTent1_5.loadFromRAR(tmpch, "tent3_05.timp");
	tent3->addFrame(frTent1_5);
	frTent1_6.loadFromRAR(tmpch, "tent3_06.timp");
	tent3->addFrame(frTent1_6);
	frTent1_7.loadFromRAR(tmpch, "tent3_07.timp");
	tent3->addFrame(frTent1_7);
	frTent1_8.loadFromRAR(tmpch, "tent3_08.timp");
	tent3->addFrame(frTent1_8);
	frTent1_9.loadFromRAR(tmpch, "tent3_09.timp");
	tent3->addFrame(frTent1_9);
	frTent1_10.loadFromRAR(tmpch, "tent3_10.timp");
	tent3->addFrame(frTent1_10);
	frTent1_11.loadFromRAR(tmpch, "tent3_11.timp");
	tent3->addFrame(frTent1_11);

	tent3->setX(-70);
	tent3->setY(180);

	tent4 = new Elemento(13);
	frTent1_0.loadFromRAR(tmpch, "tent4_00.timp");
	tent4->addFrame(frTent1_0);
	frTent1_1.loadFromRAR(tmpch, "tent4_01.timp");
	tent4->addFrame(frTent1_1);
	frTent1_2.loadFromRAR(tmpch, "tent4_02.timp");
	tent4->addFrame(frTent1_2);
	frTent1_3.loadFromRAR(tmpch, "tent4_03.timp");
	tent4->addFrame(frTent1_3);
	frTent1_4.loadFromRAR(tmpch, "tent4_04.timp");
	tent4->addFrame(frTent1_4);
	frTent1_5.loadFromRAR(tmpch, "tent4_05.timp");
	tent4->addFrame(frTent1_5);
	frTent1_6.loadFromRAR(tmpch, "tent4_06.timp");
	tent4->addFrame(frTent1_6);
	frTent1_7.loadFromRAR(tmpch, "tent4_07.timp");
	tent4->addFrame(frTent1_7);
	frTent1_8.loadFromRAR(tmpch, "tent4_08.timp");
	tent4->addFrame(frTent1_8);
	frTent1_9.loadFromRAR(tmpch, "tent4_09.timp");
	tent4->addFrame(frTent1_9);
	frTent1_10.loadFromRAR(tmpch, "tent4_10.timp");
	tent4->addFrame(frTent1_10);
	frTent1_11.loadFromRAR(tmpch, "tent4_11.timp");
	tent4->addFrame(frTent1_11);
	frTent1_12.loadFromRAR(tmpch, "tent4_12.timp");
	tent4->addFrame(frTent1_12);
	
	tent4->setX(180);
	tent4->setY(155);

	disparo1 = new Elemento(11);
	disparo2 = new Elemento(11);
	
	frTent1_0.loadFromRAR(tmpch, "disparando_00.timp");
	disparo1->addFrame(frTent1_0);
	disparo2->addFrame(frTent1_0);
	frTent1_1.loadFromRAR(tmpch, "disparando_01.timp");
	disparo1->addFrame(frTent1_1);
	disparo2->addFrame(frTent1_1);
	frTent1_2.loadFromRAR(tmpch, "disparando_02.timp");
	disparo1->addFrame(frTent1_2);
	disparo2->addFrame(frTent1_2);
	frTent1_3.loadFromRAR(tmpch, "disparando_03.timp");
	disparo1->addFrame(frTent1_3);
	disparo2->addFrame(frTent1_3);
	frTent1_4.loadFromRAR(tmpch, "disparando_04.timp");
	disparo1->addFrame(frTent1_4);
	disparo2->addFrame(frTent1_4);
	frTent1_5.loadFromRAR(tmpch, "disparando_05.timp");
	disparo1->addFrame(frTent1_5);
	disparo2->addFrame(frTent1_5);
	frTent1_6.loadFromRAR(tmpch, "disparando_06.timp");
	disparo1->addFrame(frTent1_6);
	disparo2->addFrame(frTent1_6);
	frTent1_7.loadFromRAR(tmpch, "disparando_07.timp");
	disparo1->addFrame(frTent1_7);
	disparo2->addFrame(frTent1_7);
	frTent1_8.loadFromRAR(tmpch, "disparando_08.timp");
	disparo1->addFrame(frTent1_8);
	disparo2->addFrame(frTent1_8);
	frTent1_9.loadFromRAR(tmpch, "disparando_09.timp");
	disparo1->addFrame(frTent1_9);
	disparo2->addFrame(frTent1_9);
	frTent1_10.loadFromRAR(tmpch, "disparando_10.timp");
	disparo1->addFrame(frTent1_10);
	disparo2->addFrame(frTent1_10);
	
	disparo1->setX(0);
	disparo1->setY(100);
	disparo2->setX(400);
	disparo2->setY(100);

	posicionInicial();
}


void JefeFase2::posicionInicial() {
	
	posicionX = 120;
	posicionY = 35;
	
	cabeza->setX(posicionX+66);
	cabeza->setY(posicionY);
	
	patas->setX(posicionX);
	patas->setY(posicionY+151);	
	
	mancha->setX(posicionX-70);
	mancha->setY(posicionY+60);
}

int JefeFase2::colision(Elemento *elem) {
	return cabeza->colision(elem);
}

int JefeFase2::colisionConJugador(Jugador *j) {
	if (cabeza->colision(j)) {
		return true;
	}
	if (tentaculoFuera) {
		if ( (tent1->colision(j)) && (tent1->getActivo()) ) {
			return true;
		}
		if ( (tent2->colision(j)) && (tent2->getActivo()) ) {
			return true;
		}
		if ( (tent3->colision(j)) && (tent3->getActivo()) ) {
			return true;
		}
		if ( (tent4->colision(j)) && (tent4->getActivo()) ) {
			return true;
		}
		if ( (disparo1->colision(j)) && (disparo1->getActivo()) ) {
			return true;
		}
		if ( (disparo2->colision(j)) && (disparo2->getActivo()) ) {
			return true;
		}
	}
	if (ataque) {
		int i;
		for (i=0; i<MAX_ALQUITRAN; i++) {
			if ( (alquitran1[i]->colision(j)) && (alquitran1[i]->getActivo()) ) {
				return true;
			}
			if ( (alquitran2[i]->colision(j)) && (alquitran2[i]->getActivo()) ) {
				return true;
			}
		}
	}
	return false;
}

int JefeFase2::addImpactosParaMorir (Audio *sonido, Juego *juego, Explosion *exp) { 
	int ipm;
	impactosParaMorir--; 
	barra->actualizarVida(impactosParaMorir);
	ipm = impactosParaMorir;
	setImpacto(1);
	if ( impactosParaMorir <= 0  ) {
		sonido->playSonido(SND_EXP_JEFES);
		juego->addPuntos(PUNTOS_JEFE3);
		juego->setHacerClip(0);		
		setActivo(0);
		exp->crearExplosionJefe2(posicionX-53, posicionY-45);
	}
	return ipm;
}

void JefeFase2::crearDisparo(DisparosEnemigos *disp_ene, Audio *son) {
	if (tiempoBloqueoDisparosEnemigos>0) {
		tiempoBloqueoDisparosEnemigos--;
	} else {
		switch (ciclos) {
			case 30:
				disp_ene->crearDisparo (NULL, DISP_RAFAGA_GR, cabeza->getCentroX(), cabeza->getY()+cabeza->getAlto(), 0, 0);
			break;
			case 80:
				disp_ene->crearDisparo (DISPARO_POMPAPEQ, DISP_DIRIGIDO, cabeza->getCentroX(), cabeza->getY()+cabeza->getAlto(), 50, 0);
			break;
			case 90:
				disp_ene->crearDisparo (DISPARO_POMPAPEQ, DISP_DIRIGIDO, cabeza->getCentroX(), cabeza->getY()+cabeza->getAlto(), 450, 0);
			break;
			case 110:
				disp_ene->crearDisparo (DISPARO_POMPAGR, DISP_DIRIGIDO, cabeza->getCentroX(), cabeza->getY()+cabeza->getAlto(), 450, 0);
			break;
			case 140:
				disp_ene->crearDisparo (DISPARO_GR_AZUL, DISP_5ABAJO, cabeza->getCentroX(), cabeza->getY()+cabeza->getAlto(), 0, 0);
			break;
			case 160:
				disp_ene->crearDisparo (DISPARO_ROT, DISP_2ABAJO, cabeza->getCentroX(), cabeza->getY()+cabeza->getAlto(), 0, 0);
			break;
			case 180:
				disp_ene->crearDisparo (DISPARO_ROT, DISP_5ABAJO, cabeza->getCentroX(), cabeza->getY()+cabeza->getAlto(), 0, 0);
			break;
			case 200:
				disp_ene->crearDisparo (DISPARO_ROT, DISP_2ABAJO, cabeza->getCentroX(), cabeza->getY()+cabeza->getAlto(), 0, 0);
			break;
			case 210:
				disp_ene->crearDisparo (DISPARO_POMPAGR, DISP_DIRIGIDO, cabeza->getCentroX(), cabeza->getY()+cabeza->getAlto(), 100, 0);
			break;
			case 240:
				disp_ene->crearDisparo (DISPARO_POMPAGR, DISP_DIRIGIDO, cabeza->getCentroX(), cabeza->getY()+cabeza->getAlto(), 550, 0);
			break;
			case 260:
				disp_ene->crearDisparo (NULL, DISP_RAFAGA_MED, cabeza->getCentroX(), cabeza->getY()+cabeza->getAlto(), 0, 0);
			break;
			case 300:
				disp_ene->crearDisparo (DISPARO_PEQ, DISP_3ABAJO, cabeza->getCentroX(), cabeza->getY()+cabeza->getAlto(), 0, 0);
			break;
			case 330:
				disp_ene->crearDisparo (DISPARO_PEQ, DISP_3ABAJO, cabeza->getCentroX(), cabeza->getY()+cabeza->getAlto(), 0, 0);
			break;
			case 360:
				disp_ene->crearDisparo (DISPARO_ROT, DISP_6LATERALES, cabeza->getCentroX(), cabeza->getY()+cabeza->getAlto(), 0, 0);
			break;
			case 380:
				disp_ene->crearDisparo (DISPARO_ROT, DISP_5ABAJO, cabeza->getCentroX(), cabeza->getY()+cabeza->getAlto(), 0, 0);
			break;
			case 390:
				disp_ene->crearDisparo (DISPARO_GR_AZUL, DISP_3ABAJO_CERRADO, cabeza->getCentroX(), cabeza->getY()+cabeza->getAlto(), 0, 0);
			break;
			case 430:
				disp_ene->crearDisparo (DISPARO_PEQ, DISP_3ABAJO, cabeza->getCentroX(), cabeza->getY()+cabeza->getAlto(), 0, 0);
			break;
			case 450:
				empezarAtaque = true;
				ciclosAtaque = 200;
			break;
			case 480:
				disp_ene->crearDisparo (DISPARO_PEQ, DISP_3ABAJO, cabeza->getCentroX(), cabeza->getY()+cabeza->getAlto(), 0, 0);
			break;
			case 500:
				disparo1->selFrame(0);
				disparo2->selFrame(0);
				ciclos = 0;
			break;
		}
	}
}


void JefeFase2::unLoad() {
	patas->unLoad();
	cabeza->unLoad();
}

void JefeFase2::iniciar() {
	impactosParaMorir = IMPACTOS_JEFE2;
	tiempoBloqueoDisparosEnemigos = 0;
	impacto = 0;
	ciclos = 0;
	activo = 1;
	
	barra->setTotalVida(IMPACTOS_JEFE2);
	
	posicionInicial();
	
	tentaculoFuera = false;
	nTentaculoFuera = 0;
	
	ataque = false;
	empezarAtaque = false;
	pararAlquitran = false;
	
	ciclosAtaque = 0;

}

void JefeFase2::sacarTentaculo() 
{ 
	tentaculoFuera = true; 
	nTentaculoFuera = rand()%4; 
	tent1->selFrame(0);
	tent2->selFrame(0);
	tent3->selFrame(0);
	tent4->selFrame(0);
	tent1->setActivo(0);
	tent2->setActivo(0);
	tent3->setActivo(0);
	tent4->setActivo(0);
	switch(nTentaculoFuera) {
		case 0:
			tent1->setActivo(1);
		break;
		case 1:
			tent2->setActivo(1);
		break;
		case 2:
			tent3->setActivo(1);
		break;
		case 3:
			tent4->setActivo(1);
		break;
	}
}

void JefeFase2::iniciarAlquitran() 
{ 
	int i;
	
	for (i=0; i<MAX_ALQUITRAN; i++) {
		alquitran1[i]->setX(disparo1->getX()+43);
		alquitran1[i]->setY(disparo1->getY()+210+(rand()%40));
		alquitran1[i]->setActivo(1);
		alquitran1[i]->setDX((rand()%3)-1);
		alquitran1[i]->setDY((rand()%9)+8);
		
		alquitran2[i]->setX(disparo2->getX()+43);
		alquitran2[i]->setY(disparo2->getY()+210+(rand()%40));
		alquitran2[i]->setActivo(1);
		alquitran2[i]->setDX((rand()%3)-1);
		alquitran2[i]->setDY((rand()%9)+8);
		
	}
}


void JefeFase2::dibujar(SDL_Surface *superficie) {

	int i;
	ciclos++;
	
	barra->dibujar(superficie);
	
	
	if (ataque) {
		if (ciclosAtaque>110) {
			if ( (ciclos%15 == 0) && (mancha->getActualFrame()<6) ) {
				mancha->selSiguienteFrame();
			}
			mancha->dibujar(superficie);
		} else {
			if (ciclos%4 == 0) {
				if (ciclosAtaque>0) {
					if (disparo1->getActualFrame()<disparo1->getFrames()-1) {
						disparo1->selSiguienteFrame();
						disparo2->selSiguienteFrame();
						// en el último frame empezamos a lanzar alquitran
						if (disparo1->getActualFrame()==disparo1->getFrames()-2) {
							iniciarAlquitran();
						}
					}
				} else {
					pararAlquitran = true;
					disparo1->selSiguienteFrame_Rebote();
					disparo2->selSiguienteFrame_Rebote();
					if (disparo1->getActualFrame()<=0) {
						ataque = false;
						pararAlquitran = false;
						for (i=0; i<MAX_ALQUITRAN; i++) {
							alquitran1[i]->setActivo(0);
							alquitran2[i]->setActivo(0);
						}
					}
				}
			}
			disparo1->dibujar(superficie);
			disparo2->dibujar(superficie);
			for (i=0; i<MAX_ALQUITRAN; i++) {
				if (alquitran1[i]->getActivo()) {
					alquitran1[i]->mover();
					alquitran1[i]->dibujar(superficie);
					// si sale de la pantalla y no tenemos que parar
					// el alquitran volvemos al principio
					if ( (alquitran1[i]->getY()>480) && (!pararAlquitran) ) {
						alquitran1[i]->setX(disparo1->getX()+43);
						alquitran1[i]->setY(disparo1->getY()+210);
						alquitran1[i]->setDX((rand()%3)-1);
						alquitran1[i]->setDY((rand()%9)+8);
					}
				}
				if (alquitran2[i]->getActivo()) {
					alquitran2[i]->mover();
					alquitran2[i]->dibujar(superficie);
					// si sale de la pantalla y no tenemos que parar
					// el alquitran volvemos al principio
					if ( (alquitran2[i]->getY()>480) && (!pararAlquitran) ) {
						alquitran2[i]->setX(disparo2->getX()+43);
						alquitran2[i]->setY(disparo2->getY()+210);
						alquitran2[i]->setDX((rand()%3)-1);
						alquitran2[i]->setDY((rand()%9)+8);
					}
				}
			}
		}
		ciclosAtaque--;
	} else {
		if (ciclos%180==0) {
			sacarTentaculo();
		}
	}


	if (ciclos%8==0) { 
		patas->selSiguienteFrame();
	}

	patas->dibujar(superficie);
	
	if (tentaculoFuera) {
		switch(nTentaculoFuera) {
			case 0:
				if (ciclos%4 == 0) {
					tent1->selSiguienteFrame_Rebote();
					if (tent1->getActualFrame()==0) {
						tentaculoFuera = false; 
					}
				}
				tent1->dibujar(superficie);
			break;	
			case 1:
				if (ciclos%4 == 0) {
					tent2->selSiguienteFrame_Rebote();
					if (tent2->getActualFrame()==0) {
						tentaculoFuera = false; 
					}
				}
				tent2->dibujar(superficie);
			break;	
			case 2:
				if (ciclos%4 == 0) {
					tent3->selSiguienteFrame_Rebote();
					if (tent3->getActualFrame()==0) {
						tentaculoFuera = false; 
					}
				}
				tent3->dibujar(superficie);
			break;	
			case 3:
				if (ciclos%4 == 0) {
					tent4->selSiguienteFrame_Rebote();
					if (tent4->getActualFrame()==0) {
						tentaculoFuera = false; 
					}
				}
				tent4->dibujar(superficie);
			break;	
		}		
	} else {
		if (empezarAtaque) {
			ataque = true;
			empezarAtaque = false;
		}
	}
	
	if (ciclos%100<8) {
		cabeza->selFrame(3);
	} else {
		cabeza->selFrame(0);
	}

	cabeza->dibujar(superficie);
	

}


/*****************************
**
** CLASE BARRADECARGA
**
******************************/

BarraDeCarga::BarraDeCarga(SDL_Surface *screen, int tc, int posX, int posY) : Elemento ()  {
	
	Frame frBorde, frBarra, frFin;
	
	char tmpch[127];
	
	sprintf (tmpch, "%s/miscelanea.tim", DATA_DIR);
	
	borde = new Elemento();
	frBorde.loadFromRAR(tmpch, "borde_barra.timp");
	borde->addFrame(frBorde);
	borde->setAlign(ALIGN_CENTER);
	borde->setVAlign(VALIGN_CENTER);
	
	barra_fin = new Elemento();
	frFin.loadFromRAR(tmpch, "barra_carga_fin.timp");
	barra_fin->addFrame(frFin);
	barra_fin->setVAlign(VALIGN_CENTER);
		
	ancho = 210;
	alto = 16;
	
	totalCarga = tc;
	estadoCarga = 1;
	
	frBarra.loadFromRAR(tmpch, "barra_carga.timp");
	addFrame(frBarra);

	setPosicion(posX, posY);
	
}

void BarraDeCarga::setPosicion(int posX, int posY) {
	dest.x = posX-105;
	dest.y = posY-8;
	borde->setX(posX);
	borde->setY(posY);
	barra_fin->setY(posY);
}

void BarraDeCarga::dibujar(SDL_Surface *screen) {
	src.x = 0;
	src.y = 0;
	src.w = (int)ancho*estadoCarga/totalCarga;
	src.h = alto;
	borde->dibujar(screen);
	
	SDL_BlitSurface (sprite[actualFrame].img, &src, screen, &dest);

	if (estadoCarga<totalCarga) {
		barra_fin->setX(dest.x+((int)ancho*estadoCarga/totalCarga));
		barra_fin->dibujar(screen);
	}
}

/*****************************
**
** CLASE BIOPIC
**
******************************/

BioPic::BioPic()  : Enemigo () {
	iniciar();
}

BioPic::BioPic(int nf)  : Enemigo (nf) {
	iniciar();
}

void BioPic::crearDisparo(DisparosEnemigos *disp_ene) {
	if (tiempoBloqueoDisparosEnemigos>0) {
		tiempoBloqueoDisparosEnemigos--;
	} else {
		if ( (biopicFuera) && (tiempoEsperaEntrada<0) ) {
			if (ciclos%40==0) {
				disp_ene->crearDisparo (DISPARO_GR, DISP_8, getCentroX(), getCentroY()-30, 0, 0);
			}
			if (ciclos%55==0) {
				disp_ene->crearDisparo (DISPARO_GR_AZUL, DISP_8, getCentroX(), getCentroY()-30, 0, 0);
			}
		}
	}
}

void BioPic::iniciar () {
	activo = 0;
	ciclos = 0;
	numDisparos = 0;
	disparoActual = -1;
	mostrar = 0;
	biopicFuera = 0;
	tiempoBioPicFuera = TIEMPOFIESTAFINAL3;
	tiempoEsperaEntrada = 50;
	actualFrame = 0;
	tiempoBloqueoDisparosEnemigos = 0;
}


void BioPic::dibujar (SDL_Surface *superficie) {
	SDL_Rect dest;
	dest.x = (int)posX;
	dest.y = (int)posY;
	ciclos++;

	
	if (mostrar == 1) {
		tiempoBioPicFuera--;
		if (tiempoBioPicFuera < 0) {
			tiempoBioPicFuera = TIEMPOFIESTAFINAL3;
			actualFrame = 4;
			ocultarBioPic();
		}
	}
	
	if (mostrar==1) {
		if (tiempoEsperaEntrada<0) {
			if (ciclos%4==0) {
				actualFrame++;
				if (actualFrame>18) {
					actualFrame = 5;
				}
			}
		} else {
			tiempoEsperaEntrada--;
		}
	} else {
		if (ciclos%4==0) {
			actualFrame--;
			if (actualFrame<0) {
				actualFrame = 0;
				biopicFuera = 0;
				tiempoEsperaEntrada = 50;
			}
		}				
	}
	
	if ( (biopicFuera) && (tiempoEsperaEntrada<0) ) {
		setFantasma(0);
		SDL_BlitSurface (sprite[actualFrame].img, NULL, superficie, &dest);
	} else {
		setFantasma(1);
	}
}


/*****************************
**
** CLASE ELCANO
**
******************************/

ElCano::ElCano()  : Elemento () {
}


void ElCano::cargarSprites() {
	char tmpch[127];
	
	sprintf (tmpch, "%s/elcano.tim", DATA_DIR);
	
	Frame frElCano;
	frElCano.loadFromRAR(tmpch, "elcano.timp");
	addFrame(frElCano);
	
	rampa = new Elemento();
	Frame frRampa;
	frRampa.loadFromRAR(tmpch, "rampa.timp");
	
	rampa->addFrame(frRampa);
	
	
}

void ElCano::mover() {
	ciclos++;
	addY(velocidad);
	rampa->setY(getY() + 163);
}

void ElCano::iniciar() {
	ciclos = 0;
	setX(-30);
	setY(-getAlto());
	setActivo(0);
	setVelocidad(3);
	rampaAbierta = false;
	
	rampa->setX(getX() + 128 - 72);
	rampa->setY(getY() + 163);//184
}

void ElCano::dibujarRampa (SDL_Surface *superficie) {
	if (rampaAbierta) {
		if (getX() + 128>rampa->getX())
			rampa->addX(2);
		rampa->dibujar(superficie);
	} else {
		if (getX() + 128 - 72 < rampa->getX())
			rampa->addX(-2);
		rampa->dibujar(superficie);
	}		
}

void ElCano::dibujar (SDL_Surface *superficie) {

	SDL_Rect dest;
	dest.x = (int)posX;
	dest.y = (int)posY;
	
	SDL_SetAlpha(sprite[actualFrame].img, SDL_SRCALPHA, alpha);
	SDL_BlitSurface (sprite[actualFrame].img, NULL, superficie, &dest);

}


/*****************************
**
** CLASE BARRAVIDAENEMIGO
**
******************************/

BarraVidaEnemigo::BarraVidaEnemigo(SDL_Surface *screen, char texto[32], int tv, int posX, int posY)  {
	
	totalVida = tv;
	estadoVida = tv;
	
	rDest.x = posX-30;
	rDest.y = posY-4;
	rOr.x = 0;
	rOr.y = 0;
	rOr.w = 60;
	rOr.h = 8;
	
	char tmpch[127];
	Frame frLimite, frVida, frTxt;
	
	txt = new Elemento();
	SDL_Surface *surface = crearTextoConSombra(screen, texto); 
	frTxt.loadSurface(surface);
	txt->addFrame(frTxt);
	txt->setAlign(ALIGN_RIGHT);
	txt->setVAlign(VALIGN_CENTER);
	txt->setX( posX-35 );
	txt->setY(posY);
	

	sprintf (tmpch, "%s/miscelanea.tim", DATA_DIR);
	
	frVida.loadFromRAR(tmpch, "barra_vida.timp");
	surface_vida = frVida.img;
	
	limite = new Elemento();	
	frLimite.loadFromRAR(tmpch, "barra_limite.timp");
	limite->addFrame(frLimite);
	limite->setAlign(ALIGN_CENTER);
	limite->setVAlign(VALIGN_CENTER);
	limite->setX(posX);
	limite->setY(posY);
	
}

void BarraVidaEnemigo::setPosicion(int posX, int posY) {
	rDest.x = posX-30;
	rDest.y = posY-4;
}


void BarraVidaEnemigo::dibujar(SDL_Surface *screen) {
	
	txt->dibujar(screen);
	limite->dibujar(screen);
	rOr.w = estadoVida * 60 / totalVida;
	SDL_BlitSurface (surface_vida, &rOr, screen, &rDest);
	
	
}

SDL_Surface* BarraVidaEnemigo::crearTextoConSombra(SDL_Surface *screen, char texto[127]) { 
	SDL_Surface *surface_sombra, *surface;
	
	Fuente *fuente = new Fuente();

	surface_sombra = fuente->escribirTexto(0, 13, texto, 0, 0, 0, 0, 0, 30);
	surface = fuente->escribirTexto(0, 13, texto, 255, 230, 23, 0, 0, 0);

	SDL_Surface* surface_texto = SDL_CreateRGBSurface (screen->flags,
									surface_sombra->w+4, surface_sombra->h+4, screen->format->BitsPerPixel,
									screen->format->Rmask, screen->format->Gmask,
									screen->format->Bmask,
									0);

	int colorkey = SDL_MapRGB(surface_texto->format, 240, 240, 240);
	SDL_FillRect(surface_texto, NULL, colorkey);
	SDL_SetColorKey(surface_texto, SDL_SRCCOLORKEY| SDL_RLEACCEL, colorkey);

	SDL_Rect dest;
	dest.x = 3;
	dest.y = 3;
	dest.h = surface->h+3;
	dest.w = surface->w+3;
	SDL_BlitSurface(surface_sombra, NULL, surface_texto, &dest);
	dest.x = 1;
	dest.y = 1;
	dest.h = surface->h+1;
	dest.w = surface->w+1;
	SDL_BlitSurface(surface_sombra, NULL, surface_texto, &dest);
	dest.x = 2;
	dest.y = 2;
	dest.h = surface->h+2;
	dest.w = surface->w+2;
	SDL_BlitSurface (surface, NULL, surface_texto, &dest);
	SDL_FreeSurface (surface_sombra);
	SDL_FreeSurface (surface);
	return surface_texto;
}



/*****************************
**
** CLASE MAPAAGUA
**
******************************/

 MapaAgua::MapaAgua() {
	maxmap = 468;
	indice = maxmap - 169;
	indice_in = 0;

	numTilesActual = 11;
	 
	int i;
	for ( i=0; i<=numTilesActual; i++ ) {
		suelo[i]= new Elemento();
	}
	
	velocidad = 1;

 }
 
 
void MapaAgua::dibujarMapa (SDL_Surface *screen) {
	int i,j,x,y,t;
	int dim = 40;
	int tiles_ancho = 13;
	int tiles_alto = 13;

	indice_in+=velocidad;
	if (indice_in >= dim) {
		indice_in = 0;
		indice -= tiles_ancho;
	}
	
	if (indice <= 0) {
		indice = maxmap-169;
		indice_in = 0;
	}
	int cont=0;
	for (i=0; i<tiles_alto; i++) {
		for (j=0; j<tiles_ancho; j++) {
			t = mapa[indice+(i*tiles_ancho+j)];
			cont++;
			x = j * dim;
			y = (i-1) * dim + indice_in;
			suelo[t]->selFrame(0);
			suelo[t]->setX(x);
			suelo[t]->setY(y);
			suelo[t]->dibujar(screen);
			
		}		
	}

}


char* MapaAgua::BufferFromRar(char *fileRar, char *file)
{
  char *data_ptr;
  unsigned long data_size;

  if(urarlib_get(&data_ptr, &data_size, file, fileRar, "password")) 
  {
	return data_ptr;
    //if(data_ptr != NULL) free(data_ptr);
  } else 
  { 
    printf("Error al descomprimir!\n"); 
  }
  return data_ptr;
}

void MapaAgua::Cargar (SDL_Surface *screen) {
	Frame frSuelo[MAXTILES];
	char tmpfile[50];
	char rarMapa[128];
	int i;
	// cargar mapa
	
	sprintf (rarMapa, "%s/mapaagua.tma", DATA_DIR);
	
	mapa = BufferFromRar(rarMapa, "mapa.map");
	
	for (i=0; i<=numTilesActual; i++) {
		sprintf (tmpfile, "tiles\\tile%d.timb", i);
		frSuelo[i].loadFromRAR(rarMapa, tmpfile);
		suelo[i]->addFrame(0, frSuelo[i]);
	}
	
}



void MapaAgua::iniciar () {
	// empezar el mapa desde el principio
	
	indice = maxmap - 169;  //indice = MAXMAP - 169;
	indice_in = 0;
		

	setVelocidad(VEL_MAPA);
	
}

void MapaAgua::unLoad () {
	for (int i=0; i<=numTilesActual; i++) {
		suelo[i]->unLoad();
	}
}
